#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright © 2005-2013 Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from math import floor

def MJD_convert(year, month, day):

	if (month == 1) or (month == 2):
	    l = 1
	else:
	    l = 0
	return 14956 + day + (floor((year - l) * 365.25)) + (floor((month + 1 + l * 12) * 30.6001))



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright © 2004-2013, Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils import *

######################################################################
class program_association_section(Section):

    table_id = 0x00
    
    section_max_size = 1024

    def pack_section_body(self):
    
        # pack program_loop_item
        pl_bytes = string.join(
            map(lambda x: x.pack(),
                self.program_loop),
            "")

        self.table_id_extension = self.transport_stream_id # ???
	self.private_indicator = 0

        fmt = "!%ds" % (len(pl_bytes))
        return pack(fmt,
            pl_bytes
            )

######################################################################
class program_loop_item(DVBobject):

    def pack(self):
    
        # pack program_loop_item
        fmt = "!HH"
	return pack(fmt,
	    self.program_number,
	    0xE000 | (self.PID & 0x1FFF),
	)



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright  2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
import pprint

######################################################################
def CDR(s, alignment = 4, gap_byte = 0xFF):
    """If necessary, append to 's' a trailing NUL byte
    and fill with 'gap_byte' until properly aligned.
    """

    if len(s) % alignment == 0 and s[-1] in ("\x00", "\xFF"):
        return s
    s = s + "\x00"

    while len(s) % alignment:
        s = s + chr(gap_byte)
    return s					    
					    
######################################################################
class DVBobject:
    """The base class for many protocol data units.

    Basically it provides functionality similar to a C-struct.
    Members are set via keyword arguments either in the constructor
    or via the set() method. Other (rather static) attributes
    can be defined as (sub-)class attributes.

    Subclasses must implement a 'pack()' method which returns
    the properly packed byte string.

    Attributes may come from the following sources:

    1. (Static) class attributes;

    2. Keyword arguments given in the constructor;

    3. Keyword arguments given in the 'set()' method;

    4. Direct assignment to instance attributes (NOT recommended).
    """

    #
    # Default attribute value.
    # Subclasses can do that, too!
    #
    ISO_639_language_code = "deu"

    def __init__(self, **kwargs):
        """Initialize instance attributes from keyword arguments.
        """
        apply(self.set, (), kwargs)

    def set(self, **kwargs):
        """Add (more) instance attributes from keyword arguments.
        """
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __repr__(self):
        """Used for debugging."""
        def hilite(s):
            return "### %s ###" % s
        return pprint.pformat(
            (hilite(self.__class__.__name__),
             self.__dict__))

    def dump(self):
        """Print a simple hexdump of this object to stdout.
        """
        BYTES_PER_LINE = 16
        bytes = self.pack()
        i = 0
        for byte in bytes:
            if i % BYTES_PER_LINE == 0:
                if i: print             # start on a fresh line...
                print "%04x " % i,
            print "%02X" % ord(byte),
            i = i+1
        print                           # dump is done => NL
        
    def test(self):
        """Used for debugging."""
        if not self.__dict__:
            self.sample()
        self.dump()
        print self




# This file is part of the dvbobjects library.
# 
# Copyright 2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import os
import string
from dvbobjects.utils import *
from dvbobjects.MHP.Descriptors import content_type_descriptor

######################################################################
class Message(DVBobject):

    magic = "BIOP"                      # ISO
    biop_version_major = 0x01           # ISO
    biop_version_minor = 0x00           # ISO
    byte_order = 0x00                   # DVB
    message_type = 0x00                 # ISO
    serviceContextList = ""             # MHP

    def pack(self):

        # sanity check
        assert self.byte_order      == 0x00 # DVB
        assert len(self.objectKind) == 0x04, `self.objectKind` # DVB

        MessageSubHeader_FMT = (
            "!"
            "B"
	    "L"                      # objectKey
            "L"
	    "%ds"                      # objectKind
            "H"
	    "%ds"                      # objectInfo
            "B"
	    "%ds"                      # serviceContextList
            ) % (
            len(self.objectKind),
            len(self.objectInfo),
            len(self.serviceContextList),
            )
        
        MessageSubHeader = pack(
            MessageSubHeader_FMT,
	    4,
	    self.objectKey,
            len(self.objectKind),         
	    self.objectKind,
            len(self.objectInfo),         
	    self.objectInfo,
	    len(self.serviceContextList),
	    self.serviceContextList,
            )
        
        messageBody = self.messageBody()

        message_size = (
            len(MessageSubHeader)
            + 4                         # messageBody_length field
            + len(messageBody)
            )
        
        MessageHeader = pack (
            "!"
            "4s"                        # magic
            "B"                         # biop_version_major
            "B"                         # biop_version_minor
            "B"                         # byte_order
            "B"                         # message_type
            "L"                         # message_size
            ,                           # end of FMT ;-)
            self.magic, 
            self.biop_version_major,
            self.biop_version_minor,
            self.byte_order,
            self.message_type,
            message_size,               # computed
            )
        
        FMT = ("!"
               "%ds"                    # MessageHeader
               "%ds"                    # MessageSubHeader
               "L"                      # messageBody_length
               "%ds"                    # messageBody
               ) % (
            len(MessageHeader),
            len(MessageSubHeader),
            len(messageBody)
            )
        
        return pack(
            FMT,                        # see above
            MessageHeader,
            MessageSubHeader,
            len(messageBody),           # messageBody
            messageBody,
            )

######################################################################
class FileMessage(Message):

    objectKind = CDR("fil")             # DVB

    def __init__(self, **kwargs):

        # Initialize SuperClass
        apply(Message.__init__, (self,), kwargs)

        # Initialize standard attributes
        self.set(
            objectInfo = pack("!LL", 0, self.contentSize),
            )

        # Maybe we're playing MHP...
        try:
            content_type = self.content_type
        except AttributeError:
            content_type = None         # i.e: UNKNOWN

            
        # if we know a content_type, add descriptor...
        if content_type:
            ctd = content_type_descriptor(content_type=content_type)
            self.objectInfo = self.objectInfo + ctd.pack()

    def messageBody(self):
        # Retrieve the file message body (i.e. file content)
        content = open(self.PATH).read()
        content_length = len(content)
        assert content_length == self.contentSize, self.PATH

        return pack("!L%ds" % content_length,
                    content_length,
                    content,
                    )

######################################################################
class StreamEventMessage(Message):

    objectKind = CDR("ste")             # DVB    
    objectInfo = ""

    def __init__(self, **kwargs):

        # Initialize SuperClass
        apply(Message.__init__, (self,), kwargs)
	
	info_t = pack("!BLLBBB" , 0, 0, 0, 0, 0, 1)
	# Hard coded DSM::Stream::Info_T.pack() for do it now
	event_names = open(self.PATH + "/.ename").read()
	self.objectInfo =  info_t + event_names

    def messageBody(self):

        # stream event do it now: BIOP.program_use_tap (id undefined) + eventIds
	taps = open(self.PATH + "/.tap").read()
	event_ids = open(self.PATH + "/.eid").read()
	return taps + event_ids


######################################################################
class DirectoryMessage(Message):

    objectKind = CDR("dir")             # DVB
    objectInfo = ""                     # MHP

    def __init__(self, **kwargs):

        # Initialize SuperClass
        apply(Message.__init__, (self,), kwargs)

        self.bindings_count = len(self.bindings)

        assert self.bindings_count < 512 # MHP

    def messageBody(self):
        separator = ""                  # for debugging
        bindings = "%s%s%s" % (
            separator,
            string.join(map(lambda binding: binding.pack(),
                            self.bindings),
                        separator),
            separator)

        FMT = (
            "!"
            "H"                         # bindings_count
            "%ds"                       # bindings
            ) % (len(bindings))

        messageBody = pack(
            FMT,
            len(self.bindings),
            bindings,
            )

        return messageBody

######################################################################
class ServiceGatewayMessage(DirectoryMessage):

    objectKind = CDR("srg")             # DVB

######################################################################
class ServiceGatewayInfo(DVBobject):

    userInfo = ""                       # MHP

    def pack(self):
        FMT = ("!"
               "%ds"                    # IOP::IOR
               "B"                      # downloadTaps_count
               "B"                      # serviceContextList_count
               "H%ds"                   # userInfo
               ) % (
            len(self.srg_ior),
            len(self.userInfo),
            )
        
        return pack(FMT,
                    self.srg_ior,       # IOP::IOR
                    0,                  # downloadTaps_count
                    0,                  # serviceContextList_count
                    len(self.userInfo),
                    self.userInfo,
                    )



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright 2011-2013 Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils import *
from dvbobjects.DVB.Descriptors import *

######################################################################
class selection_information_section(Section):

    table_id = 0x7F
    
    section_max_size = 4092

    def pack_section_body(self):

        self.table_id_extension = 0xFFFF
        self.private_indicator = 1
        self.section_number = 0
        self.last_section_number = 0
    
        ti_bytes = string.join(
            map(lambda x: x.pack(),
                self.transmission_info_loop),
            "")

        sl_bytes = string.join(
            map(lambda x: x.pack(),
                self.service_loop),
            "")

        fmt = "!H%ds%ds" % (len(ti_bytes), len(sl_bytes))
        return pack(fmt,
	    0xF000 | len(ti_bytes),
            ti_bytes,
            sl_bytes
            )

######################################################################
class service_loop_item(DVBobject):

    def pack(self):
    
        sdl_bytes = string.join(
            map(lambda x: x.pack(),
                self.service_descriptor_loop),
            "")

        fmt = "!HH%ds" % len(sdl_bytes)
        return pack(fmt,
                    self.service_ID,
                    0x8000 | (self.running_status << 13) |  (len(sdl_bytes) & 0x0FFF),
                    sdl_bytes,
                    )



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright © 2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils import *
from dvbobjects.DVB.Descriptors import *
from dvbobjects.MPEG.Descriptors import *

######################################################################
class program_map_section(Section):

    table_id = 0x2
    
    section_max_size = 1024

    def pack_section_body(self):
    
        # pack program_info_descriptor_loop
        pidl_bytes = string.join(
            map(lambda x: x.pack(),
                self.program_info_descriptor_loop),
            "")

        # pack stream_loop
        pl_bytes = string.join(
            map(lambda x: x.pack(),
                self.stream_loop),
            "")

        self.table_id_extension = self.program_number
	self.private_indicator = 0

        fmt = "!HH%ds%ds" % (len(pidl_bytes), len(pl_bytes))
        return pack(fmt,
	    0xE000 | (self.PCR_PID & 0x1FFF),
            0xF000 | (len(pidl_bytes) & 0x0FFF),
            pidl_bytes,
            pl_bytes,
            )

######################################################################
class stream_loop_item(DVBobject):

    def pack(self):
    
        # pack elementary_stream_info_descriptor_loop
        esidl_bytes = string.join(
            map(lambda x: x.pack(),
                self.element_info_descriptor_loop),
            "")

        fmt = "!BHH%ds" % len(esidl_bytes)
        return pack(fmt,
                    self.stream_type,
                    0xE000 | (self.elementary_PID & 0x1FFF),
                    0xF000 | (len(esidl_bytes) & 0x0FFF),
                    esidl_bytes,
                    )



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright 2006-2013 Lorenzo Pallara
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.DSMCC.Descriptors import *

######################################################################
class stream_event_section(Section):
    
    table_id = 0x3d
    
    section_max_size = 4096

    def pack_section_body(self):

        self.table_id_extension = self.event_id
	self.segment_last_section_number = self.last_section_number
	self.last_table_id = self.table_id
	
        # pack event_loop
        stel_bytes = string.join(
            map(lambda x: x.pack(),
                self.stream_event_descriptor_loop),
            "")

        fmt = "!%ds" % len(stel_bytes)
        return pack(fmt,
            stel_bytes,
            )




#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright  2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from dvbobjects.utils import *
from dvbobjects.MPEG.Section import Section

######################################################################
class TransactionId(DVBobject):

    originator = 0x02                   # DVB

    def __init__(self, **kwargs):
        # Initialize SuperClass
        apply(DVBobject.__init__, (self,), kwargs)

        # debugging
        self.HEX = "0x%08X" % self.__int__()
        self.originator = self.originator

    def __int__(self):
        return (
            self.originator << 30
            | (self.version << 16)
            | (self.identification << 1)
	    | (self.updateFlag & 0x1)
            )

######################################################################
class DSMCCmessage(Section):

    private_indicator = 0 

    protocolDiscriminator = 0x11        # DSMCC
    dsmccType = 0x03                    # DSMCC U-N Message
    adaptationLength = 0x00             # ???

    compatibilityDescriptor = ""        # DVB
    privateData = None

    def __init__(self, **kwargs):
        # Initialize SuperClass
        apply(Section.__init__, (self,), kwargs)

        # debugging

    def pack_section_body(self):
        messageBody = self.pack_message_body()
        messageLength = len(messageBody) # +- something
        FMT = ("!"                      # Network Byte Order
               "B"                      # protocolDiscriminator
               "B"                      # dsmccType
               "H"                      # messageId
               "L"                      # transactionId
               "B"                      # reserved
               "B"                      # adaptationLength
               "H"                      # messageLength
               "%ds"                    # messageBody
               ) % messageLength

        return pack(FMT,
                    self.protocolDiscriminator,
                    self.dsmccType,
                    self.messageId,
                    self.transactionId,
                    0xFF,               # reserved
                    self.adaptationLength,
                    messageLength,
                    messageBody,
                    )

######################################################################
class DownloadServerInitiate(DSMCCmessage): # DSI

    # MPEG Section attributes
    version_number = 0
    section_number = 0
    last_section_number = 0

    table_id = 0x3B                     # DSMCC
    messageId = 0x1006                  # DSMCC

    serverId = "\xFF" * 20              # DVB / MHP

    def __init__(self, **kwargs):
        # Initialize SuperClass
        apply(DSMCCmessage.__init__, (self,), kwargs)

        # debugging

    def pack_message_body(self):
        self.table_id_extension = (0x0000FFFF & int(self.transactionId))
        privateDataBytes = self.privateData.pack()

        FMT = ("!"                      # Network Byte Order
               "%ds"                    # serverId
               "H%ds"                   # compatibilityDescriptor()
               "H%ds"                   # privateDataBytes
               ) % (
            len(self.serverId),
            len(self.compatibilityDescriptor),
            len(privateDataBytes),
            )


        return pack(
            FMT,
            self.serverId,
            len(self.compatibilityDescriptor),
            self.compatibilityDescriptor,
            len(privateDataBytes),
            privateDataBytes,
            )
               
        
######################################################################
class DownloadInfoIndication(DSMCCmessage): # DII

    # MPEG Section attributes
    version_number = 0
    section_number = 0
    last_section_number = 0

    table_id = 0x3B                     # DSMCC
    messageId = 0x1002                  # DSMCC

    windowSize = 0                      # DVB
    ackPeriod = 0                       # DVB
    tCDownloadWindow = 0                # DVB
    tCDownloadScenario = 0              # ???

    def __init__(self, **kwargs):
        # Initialize SuperClass
        apply(DSMCCmessage.__init__, (self,), kwargs)

        # debugging

    def pack_message_body(self):
        self.table_id_extension = (0x0000FFFF & int(self.transactionId))

        if self.privateData == None:
            privateDataBytes = ""

        miiBytes = self.moduleInfoIndication.pack()

        fmt = ("!"                      # Network Byte Order
               "L"                      # downloadId
               "H"                      # blockSize
               "B"                      # windowSize
               "B"                      # ackPeriod
               "L"                      # tCDownloadWindow
               "L"                      # tCDownloadScenario
               "H%ds"                   # compatibilityDescriptor
               "%ds"                    # miiBytes
               "H%ds"                   # privateDataBytes
               ) % (
            len(self.compatibilityDescriptor),
            len(miiBytes),
            len(privateDataBytes),
            )
        
        return pack(fmt,
                    self.downloadId,
                    self.blockSize,
                    self.windowSize,
                    self.ackPeriod,
                    self.tCDownloadWindow,
                    self.tCDownloadScenario,
                    len(self.compatibilityDescriptor),
                    self.compatibilityDescriptor,
                    miiBytes,
                    len(privateDataBytes),
                    privateDataBytes,
                    )
 
######################################################################
class DownloadDataBlock(DSMCCmessage): # DDB

    table_id = 0x3C                     # DSMCC
    messageId = 0x1003                  # DSMCC

    def pack_message_body(self):
        # section attributes
        self.table_id_extension = self.moduleId
        self.version_number = self.moduleVersion % 32

        # override transactionId with downloadId
        # i.e. dsmccMessageHeader := dsmccDownloadHeader

        self.transactionId = self.downloadId

        fmt ="!HBBH%ds" % len(self.data_block)
        return pack(fmt,
                    self.moduleId,
                    self.moduleVersion,
                    0xFF,
                    self.blockNumber,
                    self.data_block
                    )
 



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright  2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils import *
from dvbobjects.MPEG.Descriptor import Descriptor


######################################################################
class GroupInfoIndication(DVBobject):

    privateData = ""

    def __init__(self, superGroup):

        self.numberOfGroups = len(superGroup.groups)

        self.groupInfos = []
        for group in superGroup.groups:
            self.groupInfos.append(group.getGroupInfo())

    def pack(self):
        infos = string.join(map(lambda i: i.pack(),
                                self.groupInfos),
                            "")
        FMT = ("!"
               "H"                      # NumberOfGroups
               "%ds"                    # group infos
               "H%ds"                   # privateData
               ) % (len(infos),
                    len(self.privateData))

        return pack(FMT,
                    len(self.groupInfos),
                    infos,
                    len(self.privateData),
                    self.privateData,
                    )
    
######################################################################
class GroupInfo(DVBobject):

    def pack(self):
        self.groupInfo = string.join(map(lambda d: d.pack(),
                                         self.descriptors),
                                     "")

        FMT = ("!"
               "L"                      # groupId
               "L"                      # groupSize
               "H%ds"                   # groupCompatibility
               "H%ds"                   # groupInfo
               ) % (
            len(self.groupCompatibility),
            len(self.groupInfo),
            )

        return pack(FMT,
                    self.groupId,
                    self.groupSize,
                    len(self.groupCompatibility),
                    self.groupCompatibility,
                    len(self.groupInfo),
                    self.groupInfo,
                    )

######################################################################
class ModuleInfoIndication(DVBobject):

    def __init__(self, group):

        self.numberOfModules = len(group.modules)

        self.moduleInfos = []
        for module in group.modules:
            self.moduleInfos.append(module.getModuleInfo())

    def pack(self):
        infos = string.join(map(lambda i: i.pack(),
                                self.moduleInfos),
                            "")
        FMT = ("!"
               "H"                      # NumberOfModules
               "%ds"                    # module infos
               ) % len(infos)

        return pack(FMT,
                    len(self.moduleInfos),
                    infos,
                    )
    
######################################################################
class ModuleInfo(DVBobject):

    def pack(self):
        if self.moduleInfo:
            moduleInfoBytes = self.moduleInfo.pack()
	elif self.moduleInfo == "" and len(self.descriptors) == 0:
            moduleInfoBytes = ""	    
        else:
            moduleInfoBytes = string.join(
                map(lambda d: d.pack(),
                    self.descriptors),
                "")

        FMT = ("!"
               "H"                      # moduleId
               "L"                      # moduleSize
               "B"                      # moduleVersion
               "B%ds"                   # moduleInfo
               ) % (
            len(moduleInfoBytes),
            )

        return pack(FMT,
                    self.moduleId,
                    self.moduleSize,
                    self.moduleVersion,
                    len(moduleInfoBytes),
                    moduleInfoBytes,
                    )

###################################################################
class name_descriptor(Descriptor):

    descriptor_tag = 0x02

    def bytes(self):
        self.name_length = len(self.name)
        fmt ="!%ds" % self.name_length
        return pack(fmt,
                    self.name,
                    )

###################################################################
class compressed_descriptor(Descriptor):

    descriptor_tag = 0x09

    def bytes(self):

        compression_method = 0x08

	sizeFile = "%s.size" % self.name
	items = string.split(open(sizeFile).readline())
	original_size = eval(items[0])
	# print("module size %d" % original_size)
        fmt ="!BL" 
        return pack(fmt,
	            compression_method,
                    original_size,
                    )
###################################################################
class crc32_descriptor(Descriptor):
    descriptor_tag = 0x05

    def bytes(self):
        fmt = "!L"
        return pack(fmt, self.calc_crc32)
###################################################################
class ssu_module_type_descriptor(Descriptor):
    descriptor_tag = 0x0A

    def bytes(self):
        fmt = "!B"
        return pack(fmt, self.module_type)




#! /usr/bin/env python

#
# Copyright (C) 2006-2013  Lorenzo Pallara, l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import os
import string

from dvbobjects.DSMCC.BIOP.Tap import *
from dvbobjects.utils import *


###############################
class Event_names(DVBobject):

    def pack(self):

        result = pack("!H", self.eventnames_count)
        for name in self.event_name_loop:
            name = name + "\x00"
            result = result + pack(
                "!B%ds" % len(name),
                len(name),
                name,
                )
	return result
    
###############################
class Event_ids(DVBobject):

    def pack(self):

        result = pack("!B", self.eventids_count)
        for id in self.event_id_loop:
            result = result + pack("!H", id)
	return result
    
###############################
class Taps(DVBobject):

    def pack(self):

        result = pack("!B", self.taps_count)
        for tap in self.tap_loop:
            result = result + tap.pack()
	return result
###############################
	




#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright © 2009-2013 Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from numpy import *

oddParity = array([
	0x80, 0x01, 0x02, 0x83, 0x04, 0x85, 0x86, 0x07, 0x08, 0x89, 0x8a, 0x0b, 0x8c, 0x0d, 0x0e, 0x8f, 
	0x10, 0x91, 0x92, 0x13, 0x94, 0x15, 0x16, 0x97, 0x98, 0x19, 0x1a, 0x9b, 0x1c, 0x9d, 0x9e, 0x1f, 
	0x20, 0xa1, 0xa2, 0x23, 0xa4, 0x25, 0x26, 0xa7, 0xa8, 0x29, 0x2a, 0xab, 0x2c, 0xad, 0xae, 0x2f, 
	0xb0, 0x31, 0x32, 0xb3, 0x34, 0xb5, 0xb6, 0x37, 0x38, 0xb9, 0xba, 0x3b, 0xbc, 0x3d, 0x3e, 0xbf, 
	0x40, 0xc1, 0xc2, 0x43, 0xc4, 0x45, 0x46, 0xc7, 0xc8, 0x49, 0x4a, 0xcb, 0x4c, 0xcd, 0xce, 0x4f, 
	0xd0, 0x51, 0x52, 0xd3, 0x54, 0xd5, 0xd6, 0x57, 0x58, 0xd9, 0xda, 0x5b, 0xdc, 0x5d, 0x5e, 0xdf, 
	0xe0, 0x61, 0x62, 0xe3, 0x64, 0xe5, 0xe6, 0x67, 0x68, 0xe9, 0xea, 0x6b, 0xec, 0x6d, 0x6e, 0xef, 
	0x70, 0xf1, 0xf2, 0x73, 0xf4, 0x75, 0x76, 0xf7, 0xf8, 0x79, 0x7a, 0xfb, 0x7c, 0xfd, 0xfe, 0x7f,
])

def oddparity(byte):
	byte = byte & 0x7F
	return oddParity[byte]



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright  2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# Copyright 2010, LIFIA - Facultad de Informatica - Univ. Nacional de La Plata
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils import *
from dvbobjects.MPEG.Descriptor import Descriptor

######################################################################
# Common constants.

# application_type
DVB_J_application_type = 0x0001
DVB_HTML_application_type = 0x0002

# application_control_code
DVB_J_AUTOSTART = 0x01

# protocol_id
MHP_OC_protocol_id = 0x0001

######################################################################
class application_descriptor(Descriptor):
    
    descriptor_tag = 0x00
    application_profiles_length = 5

    def bytes(self):
        tp_labels = string.join(
            map(lambda label, self=self: pack("!B", label),
                self.transport_protocol_labels),
            "")

        self.flags = (
            0x00
            | (self.service_bound_flag << 7)
            | (self.visibility << 5)
            | 0x1F
            )

        fmt = "!BH5B%ds" % len(tp_labels)
        return pack(fmt,
                    self.application_profiles_length,
                    self.application_profile,
                    self.version_major,
                    self.version_minor,
                    self.version_micro,
                    self.flags,
                    self.application_priority,
                    tp_labels,
                    )

######################################################################
class application_name_descriptor(Descriptor):

    descriptor_tag = 0x01

    def bytes(self):
        fmt = "!3sB%ds" % len(self.application_name)
        return pack(fmt,
                    self.ISO_639_language_code,
                    len(self.application_name),
                    self.application_name,
                    )

######################################################################
class transport_protocol_descriptor(Descriptor):

    descriptor_tag = 0x02

    def bytes(self):
        fmt = "!HBBB"
        return pack(fmt,
                    self.protocol_id,
                    self.transport_protocol_label,
                    0x7F | (self.remote_connection << 7),
                    self.component_tag,
                    )

######################################################################
class transport_ic_protocol_descriptor(Descriptor):

    descriptor_tag = 0x02
    protocol_id = 0x03

    def bytes(self):

        URL_extension_count = 0
	
        for url in self.URL_extensions:
            URL_extension_count = URL_extension_count + 1

        fmt = "!HBB%dsB" % len(self.URL_base)
        result = pack(fmt,
                    self.protocol_id,
                    self.transport_protocol_label,
		    len(self.URL_base),
		    self.URL_base,
		    URL_extension_count
                    )
    
        for url in self.URL_extensions:
            result = result + pack(
                "!B%ds" % len(url),
                len(url),
                url,
                )
		
	return result    


######################################################################
class application_storage_descriptor(Descriptor):

   descriptor_tag = 0x10

   def bytes(self):
       fmt = "!BBLB"
       return pack(fmt,
                   self.storage_property,
                   0x1F | (self.not_launchable_from_broadcast << 7) |
                   (self.launchable_completely_from_cache << 6) |
                   (self.is_launchable_with_older_version << 5),
                   0x80000000 | (self.version),
                   self.priority,
                   )

######################################################################
class dvb_html_application_descriptor(Descriptor):

    descriptor_tag = 0x08

    def bytes(self):

#TODO add applications_ids	
    
        fmt = "!BH%ds" % len(self.parameter)
        return pack(fmt, 1 ,1 , self.parameter)


######################################################################
class dvb_html_application_location_descriptor(Descriptor):

    descriptor_tag = 0x09

    def bytes(self):
        fmt = "!B%ds%ds" % (
            len(self.physical_root),
            len(self.initial_path),
            )

        return pack(fmt,
                    len(self.physical_root),
                    self.physical_root,
                    self.initial_path,
                    )                    

    
######################################################################
class dvb_j_application_descriptor(Descriptor):

    descriptor_tag = 0x03

    def bytes(self):
        result = ""
        
        for param in self.parameters:
            result = result + pack(
                "!B%ds" % len(param),
                len(param),
                param,
                )

        return result

######################################################################
class dvb_simple_application_location_descriptor(Descriptor):

    descriptor_tag = 0x15

    def bytes(self):
        fmt = "!%ds" % ( len(self.path) )
        return pack(fmt,
                    self.path,
                    )


######################################################################
class dvb_j_application_location_descriptor(Descriptor):

    descriptor_tag = 0x04

    def bytes(self):
        fmt = "!B%dsB%ds%ds" % (
            len(self.base_directory),
            len(self.class_path_extension),
            len(self.initial_class),
            )

        return pack(fmt,
                    len(self.base_directory),
                    self.base_directory,
                    len(self.class_path_extension),
                    self.class_path_extension,
                    self.initial_class,
                    )                    

######################################################################
class ginga_ncl_application_descriptor(Descriptor):

    descriptor_tag = 0x06

    def bytes(self):
        result = ""
        
        for param in self.parameters:
            result = result + pack(
                "!B%ds" % len(param),
                len(param),
                param,
                )

        return result

######################################################################
class ginga_ncl_application_location_descriptor(Descriptor):

    descriptor_tag = 0x07

    def bytes(self):
        fmt = "!B%dsB%ds%ds" % (
            len(self.base_directory),
            len(self.class_path_extension),
            len(self.initial_class),
            )

        return pack(fmt,
                    len(self.base_directory),
                    self.base_directory,
                    len(self.class_path_extension),
                    self.class_path_extension,
                    self.initial_class,
                    )                    

######################################################################
class content_type_descriptor(Descriptor):

    descriptor_tag = 0x72

    def bytes(self):
        return self.content_type

    def sample(self):
        self.content_type = "image/png"

###
# The following two should be somewhere else
######################################################################
#  INCOMPLETE
class additional_ginga_j_info(DVBobject):
    def __init__(self, transmission_format, document_resolution, organization_id, application_id, carousel_id):
        self.transmission_format = transmission_format
        self.document_resolution = document_resolution
        self.organization_id     = organization_id
        self.application_id      = application_id
        self.carousel_id         = carousel_id

    def bytes(self):
        fmt = "!BLHLB"
        return pack(
            fmt,
            (self.transmission_format << 6) | 0x20 | (self.document_resolution << 1),
            self.organization_id,
            self.application_id,
            self.carousel_id,
            0x1F
        )

######################################################################
class ait_identifier_info(DVBobject):
    def __init__(self, application_type, ait_version):
        self.application_type = application_type
        self.ait_version      = ait_version

    def bytes(self):
        return pack ("!HB",
            self.application_type,
            0xE0 | self.ait_version
        )




#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright  2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import os
import string

from dvbobjects.utils import *
from dvbobjects.DSMCC.Message import *
import dvbobjects.DSMCC.BIOP

from Loops import *

######################################################################
class SuperGroup(DownloadServerInitiate):
    
    def __init__(self, PATH, transactionId, version):
	self.groups =  []
	self.PATH = PATH
	self.transactionId = transactionId
	self.version_number = version
            
    def addGroup(self, group):
        self.groups.append(group)
    
    def generate(self, output_dir, SRG_IOR = None):
        for group in self.groups:
            group.generate(output_dir)

        if SRG_IOR <> None:
            self.privateData = dvbobjects.DSMCC.BIOP.ServiceGatewayInfo(
                srg_ior = SRG_IOR,
                )
        else:
            self.privateData = GroupInfoIndication(self)

        sec_file = open("%s/DSI.sec" % output_dir, "wb")
        sec_file.write(self.pack())
        sec_file.close()

######################################################################
class Group(DownloadInfoIndication):
    
    def __init__(self, PATH, transactionId, version, downloadId, blockSize):
	self.modules = []
	self.descriptors = []
	self.PATH = PATH
	self.downloadId = downloadId
	self.transactionId = transactionId
	self.version_number = version
	self.blockSize = blockSize

    def addModule(self, mod):
        self.modules.append(mod)

    def getGroupInfo(self):
        return GroupInfo(
            groupId   = self.transactionId,
            groupSize = self.groupSize,
            groupCompatibility = self.compatibilityDescriptor,
            descriptors = self.descriptors,
            )

    def generate(self, output_dir):
        self.groupSize = 0
        for module in self.modules:
            module.downloadId = self.downloadId
            module.generate(
                output_dir,
                self.transactionId,
                self.blockSize,
                )
            self.groupSize = self.groupSize + module.moduleSize

        self.__mii = mii = ModuleInfoIndication(self)
        self.moduleInfoIndication = mii
 
        path = os.path.basename(self.PATH)
        stem = os.path.splitext(path)[0]
        sec_file = open("%s/%s.sec" % (output_dir, stem), "wb")
        sec_file.write(self.pack())
        sec_file.close()

######################################################################
class Module(DVBobject):

    assocTag = None                     # i.e. NO Object Carousel
    descriptors = []

    def generate(self, output_dir, transactionId, blockSize):
        lastBlockNumber = -1
        self.moduleSize = os.stat(self.INPUT)[6]
        last_section_number = ((self.moduleSize - 1) / blockSize) % 256
        tmp = ((((self.moduleSize - 1) / blockSize) / 256) * 256) - 1

        input = open(self.INPUT, "rb")
        while 1:
            data = input.read(blockSize)
            if not data:
                break
            lastBlockNumber = lastBlockNumber + 1
            if tmp >= lastBlockNumber:
                block = DownloadDataBlock(
                    moduleId = self.moduleId,
                    moduleVersion = self.moduleVersion,
                    downloadId = self.downloadId,
                    data_block = data,
                    blockNumber = lastBlockNumber,
                    section_number = lastBlockNumber % 256,
                    last_section_number = 0xFF,
                )
            else:
                block = DownloadDataBlock(
                    moduleId = self.moduleId,
                    moduleVersion = self.moduleVersion,
                    downloadId = self.downloadId,
                    data_block = data,
                    blockNumber = lastBlockNumber,
                    section_number = lastBlockNumber % 256,
                    last_section_number = last_section_number,
                ) 

            ofn = self.__outputFilename(
                self.INPUT,
                output_dir,
                block.blockNumber)
            open(ofn, "wb").write(block.pack())

    def __outputFilename(self, input_path, output_dir, blockNumber):
        input_basename = os.path.basename(input_path)
        input_stem = os.path.splitext(input_basename)[0]
        output_basename = "%s_%06d.%s" % (
            input_stem,
            blockNumber,
            "sec",
            )
        output_path = os.path.join(
            output_dir, output_basename)
        return output_path

    def getModuleInfo(self):
        if self.assocTag <> None:
            moduleInfo = dvbobjects.DSMCC.BIOP.ModuleInfo(
                assocTag = self.assocTag,
		userInfo = self.descriptors,
                )
        else:
            moduleInfo = ""

        modInfo = ModuleInfo(
            moduleId = self.moduleId,
            moduleSize = self.moduleSize,
            moduleVersion = self.moduleVersion,
            descriptors   = self.descriptors,
            moduleInfo = moduleInfo,
            )

        return modInfo



#! /usr/bin/env python

#
# Copyright (C) 2004  Andreas Berger, berger@ftw.at
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils import *
from dvbobjects.MPEG.Descriptor import Descriptor

class ip_mac_platform_name_descriptor(Descriptor):
    
    descriptor_tag = 0x0c
    
    def bytes(self):
    
        fmt = "!3s%ds" % len(self.text_char_bytes)
    
        return pack(fmt,
                    self.ISO_639_language_code,
                    self.text_char_bytes
                    )
    
class ip_mac_platform_provider_name_descriptor(Descriptor):
    
    descriptor_tag = 0x0d
    
    def bytes(self):
    
        fmt = "!3s%ds" % len(self.text_char_bytes)
    
        return pack(fmt,
                    self.ISO_639_language_code,
                    self.text_char_bytes
                    )

class target_serial_number_descriptor(Descriptor):
    
    descriptor_tag = 0x08
    
    def bytes(self):
    
        fmt = "!%ds" % len(serial_data_bytes)
    
        return pack(fmt,
                    self.serial_data_bytes
                    )

class target_smartcard_descriptor(Descriptor):
    
    descriptor_tag = 0x06
    
    def bytes(self):
    
        fmt = "!I%ds" % len(self.private_data_bytes)
    
        return pack(fmt,
                    self.super_CA_system_id,
                    self.private_data_bytes
                    )                    

class target_MAC_address_descriptor(Descriptor):
    
    descriptor_tag = 0x07
    
    def bytes(self):
    
        fmt = "!6s%ds" % len(self.mac_addr_bytes)
    
        return pack(fmt,
                    self.mac_addr_mask,
                    self.mac_addr_bytes
                    ) 

class target_MAC_address_range_descriptor(Descriptor):
    
    descriptor_tag = 0x0e
    
    def bytes(self):
    
        fmt = "!6s%ds" % len(self.mac_addr_bytes)
    
        return pack(fmt,
                    self.mac_addr_mask,
                    self.mac_addr_bytes
                    )   

class target_IP_address_descriptor(Descriptor):
    
    descriptor_tag = 0x09
    
    def bytes(self):
    
        fmt = "!I%ds" % len(self.IPv4_addr_bytes)
    
        return pack(fmt,
                    self.IPv4_addr_mask,
                    self.IPv4_addr_bytes
                    )   
 
class target_IP_slash_descriptor(Descriptor):
    
    descriptor_tag = 0x0f
    
    def bytes(self):
    
        fmt = "!4BB"
    
        return pack(fmt,
                    self.IPv4_addr[0],
                    self.IPv4_addr[1],
                    self.IPv4_addr[2],
                    self.IPv4_addr[3],
                    self.IPv4_slash_mask
                    )   

class target_IP_source_slash_descriptor(Descriptor):
    
    descriptor_tag = 0x10
    
    def bytes(self):
    
        fmt = "!%ds" % len(self.IPv4_source_dest_bytes)
    
        return pack(fmt,
                    self.IPv4_source_dest_bytes
                    )   

class target_IPv6_address_descriptor(Descriptor):
    
    descriptor_tag = 0x0a
    
    def bytes(self):
    
        fmt = "!7s%ds" % len(self.IPv6_address_bytes)
    
        return pack(fmt,
                    self.IPv6_address_mask,
                    self.IPv6_address_bytes
                    )   

class target_IPv6_slash_descriptor(Descriptor):
    
    descriptor_tag = 0x11
    
    def bytes(self):
    
        fmt = "!%ds" % len(self.IPv6_bytes)
    
        return pack(fmt,
                    self.IPv6_bytes
                    )   

class target_IPv6_source_slash_descriptor(Descriptor):
    
    descriptor_tag = 0x12
    
    def bytes(self):
    
        fmt = "!%ds" % len(self.IPv6_source_dest_bytes)
    
        return pack(fmt,
                    self.IPv6_source_dest_bytes
                    )   

class ip_mac_stream_location_descriptor(Descriptor):
    
    descriptor_tag = 0x13
    
    def bytes(self):
    
        fmt = "!HHHHB"
    
        return pack(fmt,
                    self.network_id,
                    self.original_network_id,
                    self.transport_stream_id,
                    self.service_id,
                    self.component_tag
                    )

class isp_access_mode_descriptor(Descriptor):
    
    descriptor_tag = 0x14
    
    def bytes(self):
    
        fmt = "!B"
    
        return pack(fmt,
                    self.access_mode
                    )

class telephone_descriptor(Descriptor):
    
    descriptor_tag = 0x57
    
    def bytes(self):
    
        fmt = "!BBBB%ds%ds%ds%ds%ds" % (len(country_prefix_bytes), len(international_area_code_bytes), len(operator_code_bytes), len(national_area_code_bytes), len(core_number_bytes))
    
        return pack(fmt,
                    (0x03 << 7) & 0xC0 | (self.foreign_availability << 5) & 0x20 | self.connection_type & 0x1F,
                    (0x01 << 7) & 0x80 | (self.country_prefix_length << 5) & 0x60 | (self.international_area_code << 4) & 0x1C | self.operator_code_length & 0x07,
                    (0x01 << 7) & 0x80 | (self.national_area_code_length << 4) & 0x70 | self.core_number_length & 0x0F, 
                    country_prefix_bytes,
                    international_area_code_bytes,
                    operator_code_bytes,
                    national_area_code_bytes,
                    core_number_bytes
                    )
  
class private_data_specifier_descriptor(Descriptor):
    
    descriptor_tag = 0x5f
    
    def bytes(self):
    
        fmt = "!I"
    
        return pack(fmt,
                    self.private_data_specifier
                    )

class time_slice_fec_identifier_descriptor(Descriptor):
    
    descriptor_tag = 0x77

    def bytes(self):

    	time_slice_fec_id = 0x00;

        fmt = "!BBB"

        return pack(fmt,
                    (self.time_slicing << 7) & 0x80 | (self.mpe_fec << 5) & 0x60 | (0x03 << 3) & 0x18 | self.frame_size & 0x07,
                    self.max_burst_duration,
                    (self.max_average_rate << 4) & 0xF0 | time_slice_fec_id & 0x0F,
                    )

# FIXME: move this class to another file, it's no descriptor
class platform_id_data2(DVBobject):
  
  def pack(self):
	fmt = "!BBBBB"

	return pack(fmt,
		    (self.platform_id >> 16) & 0xFF,
		    (self.platform_id >> 8) & 0xFF,
		    self.platform_id  & 0xFF,
		    self.action_type & 0xFF,
		    (0x03 << 6) & 0xC0 | (0x00 << 5) & 0x20 | 0x01 & 0x1F
		    )
    

# FIXME: move this class to another file, it's no descriptor
class ip_mac_notification_info(DVBobject):

  def pack(self):
	  
    # pack platform id data loop
    pid_bytes = string.join(
      map(lambda x: x.pack(),
	self.platform_id_data_loop),
      "")

    platform_id_data_length = len(pid_bytes);

    fmt = "!B%ds%ds" % (platform_id_data_length, len(self.private_data_bytes))

    return pack(fmt,
		platform_id_data_length,
		pid_bytes,
		self.private_data_bytes
		)

# FIXME: move this class to another file, it's no descriptor
class platform_name(DVBobject):
  
  def pack(self):
	platform_name_length = len(self.text_char_bytes)

	fmt = "!3sB%ds" % platform_name_length

	return pack(fmt,
		    self.ISO_639_language_code,
		    platform_name_length,
		    self.text_char_bytes
		    )

# FIXME: move this class to another file, it's no descriptor
class platform_id_data(DVBobject):

  def pack(self):
	  
	pn_bytes = string.join(
	  map(lambda x: x.pack(),
	    self.platform_name_loop),
          "")

	platform_name_loop_length = len(pn_bytes)

	fmt = "!BBBB%ds" % platform_name_loop_length

	return pack(fmt,
		    (self.platform_id >> 16) & 0xFF,
		    (self.platform_id >> 8) & 0xFF,
		    self.platform_id  & 0xFF,
		    platform_name_loop_length,
		    pn_bytes
		    )
  
class linkage_descriptor(Descriptor):
    
    descriptor_tag = 0x4A;

    def bytes(self):

	if (self.linkage_type == 0x0B):
        
	  # pack platform id data loop
	  pid_bytes = string.join(
          map(lambda x: x.pack(),
	    self.platform_id_data_loop),
          "")

	  platform_id_data_length = len(pid_bytes);

	  fmt = "!BBBBBBBB%ds%ds" % (platform_id_data_length, len(self.private_data_bytes))

	  return pack(fmt,
		    (self.transport_stream_id >> 8) & 0xFF,
		    self.transport_stream_id & 0xFF,
		    (self.original_network_id >> 8) & 0xFF,
		    self.original_network_id & 0xFF,
		    (self.service_id >> 8) & 0xFF,
		    self.service_id & 0xFF,
		    self.linkage_type,
		    platform_id_data_length,
		    pid_bytes,
		    self.private_data_bytes
		    )
	else:
	  fmt = "!BBBBBBB%ds"

	  # we care only for linkage_type = 0x0B, other linkage descriptors
	  # have to be implemented according to ETSI EN 300 468 standard

	  return pack(fmt,
		    (self.transport_stream_id >> 8) & 0xFF,
		    self.transport_stream_id & 0xFF,
		    (self.original_network_id >> 8) & 0xFF,
		    self.original_network_id & 0xFF,
		    (self.service_id >> 8) & 0xFF,
		    self.service_id & 0xFF,
		    self.linkage_type,
		    private_data_bytes
		    )

class terrestrial_delivery_system_descriptor(Descriptor):

    descriptor_tag = 0x5a

    def bytes(self):
        fmt = "!LBBBL"
        return pack(fmt,
                    self.center_frequency & 0xFFFFFFFF,
		    (self.bandwidth << 5) & 0xE0 | (self.priority << 4) & 0x10 | (self.timeslice_ind << 3) & 0x08 |
		    (self.mpe_fec_ind << 2) & 0x04 | 0x03,
		    (self.constellation << 6) | (self.hierarchy_information << 3)| (self.code_rate_HP_stream),
		    (self.code_rate_LP_stream << 5) | (self.guard_interval << 3) | (self.transmission_mode << 1) | (self.other_frequency_flag),
		    0xffffffff,
                    )



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright  2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from dvbobjects.utils import *

######################################################################
class Section(DVBobject):
    """The base class of many PSI/SI Sections.

    It implements the general layout of all(?) PSI/SI
    sections with 'syntax_indicator' == 1.

    Constant Attributes:

      - 'section_syntax_indicator' = 1,

      - 'current_next_indicator' = 1.

    Attributes to be provided by subclasses:

      - 'table_id'

      - 'table_id_extension'

    Attributes to be provided by other means (e.g. using 'set()'):

      - 'version_number'

      - 'section_number'

      - 'last_section_number'

    Computed Attributes:

      - 'section_length'

      - 'crc_32'
    
    Subclasses must implement a 'pack_section_body()' method.
    """
    section_max_size = 4096
    section_syntax_indicator = 1
    private_indicator = 0
    current_next_indicator = 1

    def __sanity_check(self):
        assert self.section_syntax_indicator == 1
        assert self.current_next_indicator in (0, 1)
        assert 0 <= self.table_id <= 0xff
        assert 0 <= self.table_id_extension <= 0xffff
        assert 0 <= self.section_length <= self.section_max_size - 3
        assert 0 <= self.section_number <= 0xFF
        assert 0 <= self.last_section_number <= 0xFF

    def pack(self):
    
        body = self.pack_section_body()
        
        self.section_length = (
            5                           # section header rest
            + len(body)                 # section body
            + 4                         # CRC32
            )
        length_info_16 = (
            0xB000
            | (self.section_syntax_indicator<<15)
            | (self.private_indicator << 14)
            | (self.section_length)
            )
        version_info_8 = (
            0xC0
            | ((self.version_number & 0x01f) << 1)
            | (self.current_next_indicator)
            )

        self.__sanity_check()

        data = pack("!BHHBBB",
                    self.table_id,
                    length_info_16,
                    self.table_id_extension,
                    version_info_8,
                    self.section_number,
                    self.last_section_number,
                    ) + body

        return data + self.crc_32(data)

    def crc_32(self, data):
        crc = crc32.CRC_32(data)
        return pack("!L", crc)



#! /usr/bin/env python

#
# Copyright (C) 2013  Lorenzo Pallara, l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils import *
from dvbobjects.utils.MJD import *
from dvbobjects.MPEG.Descriptor import Descriptor

######################################################################

class rated_dimension_loop_item(DVBobject):

    def pack(self):
	
	fmt = "!BB"
	return pack(fmt,
		self.rating_dimension_j,
		0xF0 | self.rating_value & 0xF
		)

######################################################################

class rating_region_loop_item(DVBobject):

    def pack(self):
	
	self.rated_dimensions = len(self.rated_dimension_loop)
	
	data_bytes = string.join(
		map(lambda x: x.pack(),
		self.rated_dimension_loop),
		"")

	rating_description_bytes = self.rating_description_text.pack()

	fmt = "!BB%dsB%ds" % (len(data_bytes), len(rating_description_bytes))
	return pack(fmt,
		self.rating_region,
		self.rated_dimensions,
		data_bytes,
		len(rating_description_bytes),
		rating_description_bytes
		)

######################################################################
class content_advisory_descriptor(Descriptor):

    descriptor_tag = 0x87

    def bytes(self):
	self.rating_region_count = len(self.rating_region_loop)
	data_bytes = string.join(
		    map(lambda x: x.pack(),
		    self.rating_region_loop),
		    "")
		
	fmt = "!B%ds" % len(data_bytes)
	return pack(fmt,
		(0x3 << 6) | (self.rating_region_count & 0x3F),
		data_bytes,
		)

######################################################################
class AC3_audio_stream_descriptor(Descriptor):

    descriptor_tag = 0x81




#! /usr/bin/env python
 
# Copyright 2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import getopt

from dvbobjects.utils import *
from dvbobjects.utils.SpecFile import *
from dvbobjects.DVB.DataCarousel import *
from dvbobjects.DVB.Loops import *

######################################################################
def BuildCarousel(INPUT_DIR, OUTPUT_DIR):

    spec = SuperGroupSpec()
    spec.read("%s/DSI.spec" % INPUT_DIR)

    # DVB
    assert 0x0000 <= spec.transactionId & 0x0000FFFF <= 0x0001
    
    dsi = SuperGroup(
        PATH=spec.PATH,
        transactionId = spec.transactionId,
	version = spec.version,
        )

    for group in spec.groups:

        dii = Group(
            PATH = group.PATH,
            transactionId = group.transactionId,
            downloadId = group.downloadId,
            blockSize = group.blockSize,
	    version = group.version,
            )
	    
        # DVB
        assert 0x0001 <= group.transactionId & 0x0000FFFF <= 0xFFFF
	
        for mod in group.modules:
            if os.path.exists("%s.size" % mod.INPUT):
	        # print("Compressing %s") % mod.INPUT
                m = Module(INPUT="%s" % mod.INPUT,
                           moduleVersion= mod.moduleVersion,
                           moduleId = mod.moduleId,
                           assocTag = group.assocTag,
			   descriptors = [compressed_descriptor(name = mod.INPUT)],
                )
            else:
                m = Module(INPUT="%s" % mod.INPUT,
                           moduleVersion= mod.moduleVersion,
                           moduleId = mod.moduleId,
                           assocTag = group.assocTag,
                )	    
            dii.addModule(m)

    dsi.addGroup(dii)
	    
    dsi.generate(OUTPUT_DIR, spec.srg_ior)
    if spec.srg_ior == None:
        print dsi

######################################################################
OPTIONS = "h"
LONG_OPTIONS = [
    "help",
    ]

def Usage(return_code = 1):
    print ("Usage: %s"
           " [option...]"
           " <InputModuleDirectory>"
           " <OutputSectionsDirectory>") % (
        sys.argv[0])
    sys.exit(return_code)

def CheckArgs():

    try:
        opts, args = getopt.getopt(
            sys.argv[1:], OPTIONS, LONG_OPTIONS)
    except getopt.error:
        Usage()

    for opt_name, opt_val in opts:
        if opt_name in ['-h', '--help']:
            Usage(0)

    if len(args) <> 2:
        Usage()

    INPUT_DIR, OUTPUT_DIR = args
    
    BuildCarousel(INPUT_DIR, OUTPUT_DIR)

######################################################################
if __name__ == '__main__':
    CheckArgs()



#! /usr/bin/env python

from distutils.core import setup, Extension

import sys

# from an original source code by Joachim Kaeber (kaeber@gmd.de)

_ext_modules = None

if sys.platform in ['linux2', 'solaris2', 'win32']:
    _ext_modules = [ Extension('dvbobjects.utils._crc32', [ 'sectioncrc.py.c'] ), ]

setup(
    name = "dvbobjects",
    version = "0.1",
    description = "Python Package for dvb transport stream data generation (PAT, PMT, NIT, Object Carousel, ...)",
    author = "Lorenzo Pallara",
    author_email = "l.pallara@avalpa.com",
    url = "",
    
    packages = [
        'dvbobjects',
        'dvbobjects.ATSC',
        'dvbobjects.DSMCC',
        'dvbobjects.DSMCC.BIOP',
        'dvbobjects.DVB',
        'dvbobjects.MHP',
        'dvbobjects.HBBTV',
	'dvbobjects.PSI',
        'dvbobjects.MPEG',
        'dvbobjects.SBTVD',
        'dvbobjects.utils',
        ],

    ext_modules = _ext_modules
)



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright 2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from dvbobjects.utils import *

######################################################################
class _tap(DVBobject):

    BIOP_DELIVERY_PARA_USE   = 0x0016
    BIOP_OBJECT_USE          = 0x0017
    BIOP_ES_USE              = 0x0018
    BIOP_PROGRAM_USE         = 0x0019
    STR_NPT_USE              = 0x000B
    STR_STATUS_AND_EVENT_USE = 0x000C
    STR_EVENT_USE            = 0x000D
    STR_STATUS_USE           = 0x000E

    id  = 0x0000                        # MHP

######################################################################
class object_tap(_tap):

    use = _tap.BIOP_OBJECT_USE

    selector = ""                       # MHP ???

    def pack(self):
        assert self.selector == ""      # MHP ???
        
        FMT = (
            "!"
            "H"                         # id
            "H"                         # use
            "H"                         # assocTag
            "B"                         # selector_length
            )
        
        return pack(
            FMT,
            self.id,
            self.use,
            self.assocTag,
            len(self.selector),
            )

######################################################################
class elementary_stream_tap(_tap):

    use = _tap.BIOP_ES_USE

    selector = ""                       # MHP ???

    def pack(self):
        assert self.selector == ""      # MHP ???
        
        FMT = (
            "!"
            "H"                         # id
            "H"                         # use
            "H"                         # assocTag
            "B"                         # selector_length
            )
        
        return pack(
            FMT,
            self.id,
            self.use,
            self.assocTag,
            len(self.selector),
            )

######################################################################
class program_use_tap(_tap):

    use = _tap.BIOP_PROGRAM_USE

    selector = ""                       # MHP ???

    def pack(self):
        assert self.selector == ""      # MHP ???
        
        FMT = (
            "!"
            "H"                         # id
            "H"                         # use
            "H"                         # assocTag
            "B"                         # selector_length
            )
        
        return pack(
            FMT,
            self.id,
            self.use,
            self.assocTag,
            len(self.selector),
            )
	    
######################################################################
class str_event_use_tap(_tap):

    use = _tap.STR_EVENT_USE

    selector = ""                       # MHP ???

    def pack(self):
        assert self.selector == ""      # MHP ???
        
        FMT = (
            "!"
            "H"                         # id
            "H"                         # use
            "H"                         # assocTag
            "B"                         # selector_length
            )
        
        return pack(
            FMT,
            self.id,
            self.use,
            self.assocTag,
            len(self.selector),
            )

######################################################################
class delivery_para_tap(_tap):

    use = _tap.BIOP_DELIVERY_PARA_USE

    selector_length = 0x0A              # MHP
    selector_type   = 0x0001            # MHP
    
    def pack(self):
        FMT = (
            "!"
            "H"                         # id
            "H"                         # use
            "H"                         # assocTag
            "B"                         # selector_length
            "H"                         # selector_type
            "L"                         # transaction_id
            "L"                         # timeout
            )
        
        return pack(
            FMT,
            self.id,
            self.use,
            self.assocTag,
            self.selector_length,
            self.selector_type,
            int(self.transactionId),
	    int(self.timeout),
            )



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright 2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from dvbobjects.utils import *

BINDING_TYPE_NOBJECT  = 0x01
BINDING_TYPE_NCONTEXT = 0x02

######################################################################
class Binding(DVBobject):

    nameComponents_count = 1            # DVB

    def __init__(self, **kwargs):

        # Initialize SuperClass
        apply(DVBobject.__init__, (self,), kwargs)

        assert self.bindingType in (    # DVB
            BINDING_TYPE_NOBJECT,
            BINDING_TYPE_NCONTEXT)

    def pack(self):

        ior = self.IOR.pack()

        FMT = (
            "!"
            "B"                         # nameComponents_count
            "B%ds"                      # id
            "B%ds"                      # kind
            "B"                         # bindingType
            "%ds"                       # IOP::IOR()
            "H%ds"                      # objectInfo
            ) % (
            len(self.nameId),
            len(self.nameKind),
            len(ior),
            len(self.objectInfo),
            )

        return pack(
            FMT,
            self.nameComponents_count,
            len(self.nameId), 
	    self.nameId,
            len(self.nameKind), 
	    self.nameKind,
            self.bindingType,
            ior,
            len(self.objectInfo), 
	    self.objectInfo,
            )

    def __repr__(self):
        """Overrides DVBobject.__repr_, which is noooiiisy"""
        loc = self.IOR.profile.objectLocation
        mod = loc.moduleId
        key = loc.objectKey
        return `(self.nameId,
                 (mod, key),
                 )`

######################################################################
class ObjectFileBinding(Binding):

    bindingType = BINDING_TYPE_NOBJECT
    nameKind = CDR("fil")
    
    def __init__(self, **kwargs):

        # Initialize SuperClass
        apply(Binding.__init__, (self,), kwargs)

       	self.objectInfo = pack("!LL", 0, self.contentSize)
        
######################################################################
class ObjectStreamEventBinding(Binding):

    bindingType = BINDING_TYPE_NOBJECT
    nameKind = CDR("ste")
    objectInfo = ""

    def __init__(self, **kwargs):

        # Initialize SuperClass
        apply(Binding.__init__, (self,), kwargs)

######################################################################
class ContextBinding(Binding):

    bindingType = BINDING_TYPE_NCONTEXT
    nameKind    = CDR("dir")            # MHP
    objectInfo  = ""                    # MHP

    def __init__(self, **kwargs):

        # Initialize SuperClass
        apply(Binding.__init__, (self,), kwargs)




#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright 2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# Copyright 2010, LIFIA - Facultad de Informatica - Univ. Nacional de La Plata
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils import *
from dvbobjects.MPEG.Descriptor import Descriptor

######################################################################
class data_broadcast_descriptor(Descriptor):

    descriptor_tag    = 0x64
    data_broadcast_id = 0x0007
    text_chars = ""

    def bytes(self):
        self.selector_length = len(self.selector_bytes)
        self.text_length = len(self.text_chars)

        FMT = "!HBB%ds%dsB%ds" % (
            self.selector_length,
            3,                          # ISO_639_language_code
            self.text_length,
            )

        return pack(
            FMT,
            self.data_broadcast_id,
            self.component_tag,
            self.selector_length,
            self.selector_bytes,
            self.ISO_639_language_code,
            self.text_length,
            self.text_chars,
            )

    def sample(self):
        self.set(
            component_tag = 0x11,
            selector_bytes = "012345",
            text_chars = "abcde",
            )


######################################################################
class object_carousel_info(DVBobject):

    carousel_type_id   = 0x02
    transaction_id     = 0xFFFFFFFF
    time_out_value_DSI = 0xFFFFFFFF
    time_out_value_DII = 0xFFFFFFFF
    leak_rate = 0x00

    def pack(self):

        obj_loop_bytes = ""
        for obj_name_chars in self.object_names:
            object_name_length = len(obj_name_chars)
            obj_bytes = pack(
                "!3sB%ds" % object_name_length,
                self.ISO_639_language_code,
                object_name_length,
                obj_name_chars,
                )
            obj_loop_bytes = obj_loop_bytes + obj_bytes
            

        obj_loop_length = len(obj_loop_bytes)

        FMT = "!B4L%ds" % obj_loop_length
        return pack(
            FMT,
            (self.carousel_type_id << 6 | 0x63),
            self.transaction_id,
            self.time_out_value_DSI,
            self.time_out_value_DII,
            (0x0C000000 | self.leak_rate),
            obj_loop_bytes,
            )

    def sample(self):
        self.set(
            object_names = ["aaa", "bbbbbb"]
            )

######################################################################
class stream_event_descriptor(Descriptor):

    descriptor_tag = 0x1a
    
    reserved = 0x0L # TODO Check: Shouldn't this be 0xFFFFFFFEL?
    
    def bytes(self):
        self.text_length = len(self.private_data)

        FMT = "!HLL%ds" % self.text_length

        return pack(
            FMT,
            self.event_id,
            self.reserved | ( self.eventNPT >> 32), # 31 bits reserved, 1 bit NPT_reference
            self.eventNPT & 0xFFFFFFFF,     # 32 bits NPT_reference
            self.private_data,
            )

######################################################################
class stream_event_do_it_now_descriptor(stream_event_descriptor):
    
    eventNPT = 0x0
######################################################################
class stream_event_ginga_descriptor(stream_event_descriptor):

    eventNPT  = 0x0
    
    command_tag = 0x0
    sequence    = 0x0 
    finalFlag   = 0x0
    payload     = ""

    FCS         = 0x00 # TODO calculate FCS.

    def bytes(self):
        FMT = ("!"
            "B" # private data length
            "B" # command_tag
            "B" # sequence_number: 7b, final_flag: 1b
            "%ds"# private data payload
            "B" # FCS
        ) % (len(self.payload))

        self.private_data = pack(FMT,
            3+len(self.payload),
            self.command_tag,
            ((self.sequence_number & 0x7F) << 1) | (self.final_flag & 0x01),
            self.payload,
            self.FCS
        )

        # call superclass implementation.
        return stream_event_descriptor.bytes(self)

######################################################################
class npt_reference_descriptor(Descriptor):

    descriptor_tag = 0x01
    
    def bytes(self):
        self.text_length = len(self.private_data)

        FMT = ( "!"
		"B" # post_continuity_indicator: 1b, content_id: 7b
		"B" # reserved:  7b, STC_reference:  1b - most significant bit
		"L" #                STC_reference: 32b
		"L" # reserved: 31b, NPT_reference:  1b - most significant bit
		"L" #                NPT_reference: 32b
		"H" # scale_numerator  : 16b
		"H")# scale_denominator: 16b

        return pack(
            FMT,
            (self.post_continuity_indicator << 7) | (self.content_id & 0x7F), # 1 bit postcontdesc, 7 bits contentId
            0xFE | ( self.STC_reference >> 32),  # 31 bits reserved, 1 bit STC_reference
            self.STC_reference & 0xFFFFFFFF,     # 32 bits STC_reference
            0xFFFFFFFE | ( self.NPT_reference >> 32), # 31 bits reserved, 1 bit NPT_reference
            self.NPT_reference & 0xFFFFFFFF,     # 32 bits NPT_reference
            self.scale_numerator,
            self.scale_denominator
            )

######################################################################
class npt_endpoint_descriptor(Descriptor):

    descriptor_tag = 0x02
    
    def bytes(self):
        self.text_length = len(self.private_data)

        FMT = ( "!"
		"H" # reserved: 15b, NPT_start:  1b - most significant bit
		"L" #                NPT_start: 32b
		"L" # reserved: 31b, NPT_stop :  1b - most significant bit
		"L")#                NPT_stop : 32b

        return pack(
            FMT,
            0xFFFE | ( self.NPT_start >> 32), # 15 bits reserved, 1 bit NPT_start
            self.NPT_start & 0xFFFFFFFF,      # 32 bits STC_reference
            0xFFFFFFFE | ( self.NPT_stop >> 32), # 31 bits reserved, 1 bit NPT_stop
            self.NPT_stop & 0xFFFFFFFF        # 32 bits NPT_stop
            )




#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright © 2005-2013 Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils.MJD import *
from dvbobjects.ATSC.Descriptors import *
from dvbobjects.ATSC.Loops import *

######################################################################
class event_information_section(Section):
    
    table_id = 0xCB
    
    section_max_size = 4093

    def pack_section_body(self):

        self.table_id_extension = self.source_id
        self.protocol_version = 0
	
        self.num_events_in_section = len(self.event_loop)

        # pack event_loop
        el_bytes = string.join(
            map(lambda x: x.pack(),
                self.event_loop),
            "")

        fmt = "!BB%ds" % len(el_bytes)
        return pack(fmt,
	    self.protocol_version,
	    self.num_events_in_section,
            el_bytes,
            )

######################################################################
class event_loop_item(DVBobject):

    def pack(self):
    

        # pack event_descriptor_loop
        event_descriptors_bytes = string.join(
            map(lambda x: x.pack(),
                self.descriptor_loop),
            "")

	title_text_bytes = self.title_text.pack()

        fmt = "!HLBHB%dsH%ds" % (len(title_text_bytes), len(event_descriptors_bytes))
        return pack(fmt,
                    (0x3 << 14) | (self.event_id & 0x3FFF),
		    self.start_time,
		    (0x3 << 6) | ((self.ETM_location & 0x3) << 4) | ((self.length_in_seconds >> 16) & 0xF),
		    (self.length_in_seconds & 0xFFFF),
		    len(title_text_bytes),
		    title_text_bytes,
		    (0xF << 12) | (len(event_descriptors_bytes) & 0x0FFF),
		    event_descriptors_bytes,
		    )



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright 2010-2013 Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils import *
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils.DVBobject import *

######################################################################
class terrestrial_virtual_channel_section(Section):

	table_id = 0xC8
	section_max_size = 1024

	def pack_section_body(self):

		self.table_id_extension = self.transport_stream_id
	
		# pack event_loop
		cl_bytes = string.join(
			map(lambda x: x.pack(),
			self.channels_loop),
		"")
		
		adl_bytes = string.join(
			map(lambda x: x.pack(),
			self.additional_descriptors_loop),
		"")

		fmt = "!BB%dsH%ds" % (len(cl_bytes), len(adl_bytes))
		return pack(fmt,
			self.ATSC_protocol_version,
			len(self.channels_loop),
			cl_bytes,
			0xFC00 | (len(adl_bytes) & 0x3FF),
			adl_bytes
		)

######################################################################
class channel_loop_item(DVBobject):

	def pack(self):
	
		assert len(self.short_name) == 14 #bytes -> 7*16 bit, 7 utf-16 chars
	
	# pack event_descriptor_loop
		dl_bytes = string.join(
			map(lambda x: x.pack(),
			self.descriptors_loop),
			"")

		fmt = "!%dsLLHHBBHH%ds" % (len(self.short_name), len(dl_bytes))
		return pack(fmt,
			self.short_name,
			0xF0000000 | ((self.major_channel_number & 0x3FF) << 18) | ((self.minor_channel_number & 0x3FF) << 8) | (self.modulation_mode & 0xFF),
			self.carrier_frequency,
			self.channel_TSID,
			self.program_number,
			(self.ETM_location << 6) | (self.access_controlled << 5) | (self.hidden << 4) | 0xC | (self.hide_guide << 1) | 0x1,
			0xC0 | self.service_type,
			self.source_id,
			0xFC00 | (len(dl_bytes) & 0x3FF),
			dl_bytes
			)



#! /usr/bin/env python

# Copyright  2007 Andreas Berger
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils import *
from dvbobjects.DVB.Descriptors import *

######################################################################
class ip_mac_notification_section(Section):

    table_id = 0x4c
    
    section_max_size = 4096

    def pack_section_body(self):

        self.action_type = 0x01
        self.platform_id_hash = ( (self.platform_id>>16) & 0xff ) ^ ( (self.platform_id>>8) & 0xff ) ^ ( self.platform_id & 0xff )
    
        # pack platform descriptor loop
        pdl_bytes = string.join(
            map(lambda x: x.pack(),
                self.platform_descriptor_loop),
            "")

        # pack associaton_loop
        al_bytes = string.join(
            map(lambda x: x.pack(),
                self.association_loop),
            "")

	pdl_bytes_length = len(pdl_bytes)
        self.table_id_extension = self.action_type << 8 | self.platform_id_hash

        fmt = "!BBBBBB%ds%ds" % (len(pdl_bytes), len(al_bytes))
        return pack(fmt,
            (self.platform_id >> 16) & 0xFF,
            (self.platform_id >> 8) & 0xFF,
            self.platform_id & 0xFF,
            self.processing_order,
            0xF0 << 8 | (pdl_bytes_length >> 8) & 0x0F,
	    pdl_bytes_length & 0xFF,
            pdl_bytes,
            al_bytes
            )

class association_loop_item(DVBobject):
  
  def pack(self):
        # pack target descriptor loop
        tdl_bytes = string.join(
            map(lambda x: x.pack(),
                self.target_descriptor_loop),
            "")

        # pack operational descriptor loop
        odl_bytes = string.join(
            map(lambda x: x.pack(),
                self.operational_descriptor_loop),
            "")

	tdl_bytes_length = len(tdl_bytes)
	odl_bytes_length = len(odl_bytes)
	
	fmt = "!BB%dsBB%ds" % (tdl_bytes_length, odl_bytes_length)
	
	return pack(fmt,
		0xF0 << 8 | (tdl_bytes_length >> 8) & 0x0F,
		tdl_bytes_length & 0xFF,
		tdl_bytes,
		0xF0 << 8 | (odl_bytes_length >> 8) & 0x0F,
		odl_bytes_length & 0xFF,
		odl_bytes
		)
		



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright © 2009-2013 Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils import *
from dvbobjects.utils.DVBobject import *
from dvbobjects.utils.MJD import *
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils import *
from dvbobjects.DVB.Descriptors import *

######################################################################
class time_offset_section(DVBobject):
	
	def pack(self):
	
		date = MJD_convert(self.year, self.month, self.day)
		# pack service_stream_loop
		tl_bytes = string.join(
			map(lambda x: x.pack(),
			self.descriptor_loop),
			"")
		
		fmt = "!BHHBBBH%ds" % len(tl_bytes) 
		data = pack(fmt,
			0x73,
			0x7000 | ((len(tl_bytes) + 11) & 0xFFF),
			date,
			self.hour,
			self.minute,
			self.second,
			0xF000 | (len(tl_bytes) & 0xFFF),
			tl_bytes
		)
		return data + self.crc_32(data)
	
	def crc_32(self, data):
		crc = crc32.CRC_32(data)
		return pack("!L", crc)



#! /usr/bin/env python
#
# Copyright (C) 2010 Robson Tovo <robson.tovo@gmail.com>,
#                    Marco Casaroli <marco.casaroli@gmail.com>
#					 LIFIA - Facultad de Informatica - Univ. Nacional de La Plata
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils import *
from dvbobjects.utils.MJD import *
from dvbobjects.MPEG.Descriptor import Descriptor

######################################################################
class tds_frequency_item(DVBobject):

    def pack(self):
        self.freq7 = (self.freq * 7) + 1
        fmt = "!H"
        return pack(fmt, self.freq7)

class terrestrial_delivery_system_descriptor(Descriptor):

    descriptor_tag = 0xfa

    def bytes(self):
        frequency_bytes = string.join(
                map(lambda x: x.pack(), 
                self.frequencies),
                "")
            
        fmt = "!H%ds" % len(frequency_bytes)
        return pack(fmt, ((self.area_code & 0xFFF) << 4) | ((self.guard_interval & 0x3) << 2) | (self.transmission_mode & 0x3), frequency_bytes)

######################################################################
# As specified by ARIB
class partial_reception_descriptor(Descriptor):

    descriptor_tag = 0xfb

    def bytes(self):
	sid_pack = string.join(map (lambda x: pack("!H", x), self.service_ids), "")

	fmt = "!%ds" % (len(sid_pack))
        return pack(fmt, sid_pack)

######################################################################
# As specified by ARIB
class system_management_descriptor(Descriptor):

    descriptor_tag = 0xfe

    def bytes(self):
        sys_man_id = (
            (self.broadcasting_flag << 14) |
            (self.broadcasting_identifier << 8) |
            (self.additional_broadcasting_identification)
        )

        additional_id_info_pack = string.join(
            map (lambda x: pack("!B", x), self.additional_identification_bytes),
            ""
        )
        fmt = "!H%ds" % len(additional_id_info_pack)

        return pack(fmt, sys_man_id, additional_id_info_pack)

######################################################################
class data_component_descriptor(Descriptor):

    descriptor_tag = 0xfd

    def bytes(self):
        fmt = "!H%ds" % len(self.additional_data_component_info)
        return pack(fmt,
            self.data_component_id,
            self.additional_data_component_info,
        )

######################################################################
class service_id_loop_item(DVBobject):
    def pack(self):
        fmt = "!H"
        return pack(fmt, self.service_id)

class transmission_type_loop_item(DVBobject):

    def pack(self):
        num_of_service = len(self.service_id_loop)
        service_id_loop_cat = string.join(
            map(
                lambda x: x.pack(),
                self.service_id_loop
            ), ""
        )
    
        fmt = "!BB%ds" % len(service_id_loop_cat)
        return pack(fmt,
            self.transmission_type_info,
            num_of_service,
            service_id_loop_cat,
        )

class transport_stream_information_descriptor(Descriptor):

    descriptor_tag = 0xcd

    def bytes(self):

        name_len = len(self.ts_name)
        ts_type_count = len(self.transmission_type_loop)
        transmission_type_loop_cat = string.join(
            map(
                lambda x: x.pack(),
                self.transmission_type_loop
            ), ""
        )

        fmt = "!BB%ds%ds" % (name_len, len(transmission_type_loop_cat))
        return pack(fmt,
            self.remote_control_key_id,
            (name_len << 2) | ts_type_count,
            self.ts_name,
            transmission_type_loop_cat
        )

######################################################################

class video_decode_control_descriptor(Descriptor):
    descriptor_tag = 0xc8

    def bytes(self):
        return pack("!B",
            (self.still_picture_flag << 7) |
            (self.sequence_end_code_flag << 6) |
            (self.video_encode_format << 2) |
            0x03
        )

######################################################################

class ts_loop_item(DVBobject):
    #tx_type_info
    # svc_loop = []
    def pack(self):
        svc_bytes = "".join(map(lambda x: chr(x>>8 & 0xFF) + chr(x&0xFF), self.svc_loop))
        fmt = "!BB%ds" % len(svc_bytes)
        return pack(fmt, self.tx_type_info, len(self.svc_loop), svc_bytes)

class ts_info_descriptor(Descriptor):
    descriptor_tag = 0xcd
    # rc_key
    # tsname
    #ts_loop = []
    def pack(self):
        ts_loop_bytes = "".join(map(lambda x: x.pack(), self.ts_loop))
        fmt = "!BBBB%ds%ds" % (len(self.tsname), len(ts_loop_bytes))
#	print len(ts_loop_bytes)
        return pack(fmt, self.descriptor_tag, len(self.tsname) + len(ts_loop_bytes) + 2, self.rc_key, len(self.tsname)<<2 | len(self.ts_loop) & 3, self.tsname, ts_loop_bytes)
	



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright 2009-2013 Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils import *
from dvbobjects.DVB.Descriptors import *

######################################################################
class update_notification_section(Section):

    table_id = 0x4B
    
    section_max_size = 4096

    def pack_section_body(self):
    
        self.table_id_extension = self.action_type << 8 | ((self.OUI >> 16) ^ ((self.OUI >> 8) & 0xFF) ^ (self.OUI & 0xFF))
        
        # pack common_descriptor_loop
        common_bytes = string.join(
            map(lambda x: x.pack(),
                self.common_descriptor_loop),
            "")

        # pack compatibility_descriptor_loop
        compatibility_bytes = string.join(
            map(lambda x: x.pack(),
                self.compatibility_descriptor_loop),
            "")

        fmt = "!HBBH%ds%ds" % (len(common_bytes), len(compatibility_bytes))
        return pack(fmt,
    	    self.OUI >> 8,
    	    self.OUI & 0xFF,
    	    self.processing_order,
    	    0xF000 | len(common_bytes),
    	    common_bytes,
    	    compatibility_bytes,
            )

######################################################################
class unt_compatibility_descriptor_loop_item(DVBobject):

    def pack(self):
    
        # pack target_descriptor_loop
        tdl_bytes = string.join(
            map(lambda x: x.pack(),
                self.target_descriptor_loop),
            "")
            
        # pack operational descriptor_loop
        odl_bytes = string.join(
            map(lambda x: x.pack(),
                self.operational_descriptor_loop),
            "")
        
        fmt = "!%dsHH%dsH%ds" % (len(self.compatibility_descriptor), len(tdl_bytes), len(odl_bytes))
        return pack(fmt,
    		    self.compatibility_descriptor,
    		    len(tdl_bytes) + len(odl_bytes),
    		    0xF000 | len(tdl_bytes),
                    tdl_bytes,
                    0xF000 | len(odl_bytes),
                    odl_bytes,
                    )



#! /usr/bin/env python

# This file is part of the dvbobjects library and contains the main HbbTV descriptors (see the list below)
# 
# Copyright  2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# Copyright  2010, mediatvcom, Claude Vanderm
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

#Below are listed the HbbTV available, and missing descriptor of this release.
#Based on HbbTV specification (ETSI-TS 102 796) version 1.1.1
#
#available hbbtv descriptors:                     | optional missing descriptors:
#application_descriptor()                         | application_icons_descriptor()
#application_name_descriptor()                              
#transport_protocol_descriptor()          
#simple_application_location_descriptor()                                       
#application_usage_descriptor()
#simple_application_boundary_descriptor()
#external_application_authorisation_descriptor()

import string
from dvbobjects.utils import *
from dvbobjects.MPEG.Descriptor import Descriptor

######################################################################
class application_descriptor(Descriptor):
    
    descriptor_tag = 0x00
    application_profiles_length = 5

    def bytes(self):
        tp_labels = string.join(
            map(lambda label, self=self: pack("!B", label),
                self.transport_protocol_labels),
            "")

        self.flags = (
            0x00
            | (self.service_bound_flag << 7)
            | (self.visibility << 5)
            | 0x1F
            )

        fmt = "!BH5B%ds" % len(tp_labels)
        return pack(fmt,
                    self.application_profiles_length,
                    self.application_profile,
                    self.version_major,
                    self.version_minor,
                    self.version_micro,
                    self.flags,
                    self.application_priority,
                    tp_labels,
                    )
######################################################################
class application_name_descriptor(Descriptor):

    descriptor_tag = 0x01

    def bytes(self):
        fmt = "!3sB%ds" % len(self.application_name)
        return pack(fmt,
                    self.ISO_639_language_code,
                    len(self.application_name),
                    self.application_name,
                    )
######################################################################
class transport_protocol_descriptor(Descriptor):

    descriptor_tag = 0x02

    def bytes(self):
        if self.protocol_id == 0x0001:
            fmt = "!HBBB"
            return pack(fmt,
                        self.protocol_id,
                        self.transport_protocol_label,
                        0x7F | (self.remote_connection << 7),
                        self.component_tag,
                        )
                        
        if self.protocol_id == 0x0003:
            URL_extension_count = 0
            for url in self.URL_extensions:
                URL_extension_count = URL_extension_count + 1
            fmt = "!HBB%dsB" % len(self.URL_base)
            result = pack(fmt,
                        self.protocol_id,
                        self.transport_protocol_label,
                        len(self.URL_base),
                        self.URL_base,
                        URL_extension_count
                        )
            for url in self.URL_extensions:
                result = result + pack(
                    "!B%ds" % len(url),
                    len(url),
                    url,
                    )
            return result
######################################################################
class simple_application_location_descriptor(Descriptor):

    descriptor_tag = 0x15

    def bytes(self):
        fmt = "!%ds" % len(self.initial_path_bytes)
        return pack(fmt,
                    self.initial_path_bytes
                    )
######################################################################
class application_usage_descriptor(Descriptor):

    descriptor_tag = 0x16
    
    def bytes(self):
        fmt = "!B" 
        return pack(fmt,
                    self.usage_type
                    )
                    
######################################################################
class simple_application_boundary_descriptor(Descriptor):
    
    descriptor_tag = 0x17
    
    def bytes(self):
        boundary_extension_count = 0
        for boundary_extension in self.boundary_extensions:
            boundary_extension_count = boundary_extension_count + 1
        fmt = "!B"
        result = pack(fmt,
                      boundary_extension_count
                      )
        for boundary_extension in self.boundary_extensions:
            result = result + pack("!B%ds" % len(boundary_extension),
                                    len(boundary_extension),
                                    boundary_extension,
                                    )
        return result
######################################################################
class external_application_authorisation_descriptor(Descriptor):
    
    descriptor_tag = 0x05

    def bytes(self):
        result = ""
        for y in range(len(self.application_identifiers)):
            i=0
            fmt = "!LHB"
            result = result + pack(fmt,
                                    self.application_identifiers[y][i], #organisation_id,
                                    self.application_identifiers[y][i+1], # application_id,
                                    self.application_priority[y]
                                    )
        return result



#! /usr/bin/env python

#
# Copyright (C) 2013  Lorenzo Pallara, l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils import *

######################################################################

class segment_loop_item(DVBobject):

    def pack(self):
	
	fmt = "!BBB%ds" % len(self.compressed_string)
	return pack(fmt,
		self.compression_type,
		self.mode,
		len(self.compressed_string),
		self.compressed_string,
		)



######################################################################

class string_loop_item(DVBobject):

    def pack(self):
	assert len(self.ISO639_language_code) == 3
	
	data_bytes = string.join(
		map(lambda x: x.pack(),
		self.segment_loop),
		"")

	self.number_segments = len(self.segment_loop)
	
	fmt = "!%dsB%ds" % (len(self.ISO639_language_code), len(data_bytes))
	return pack(fmt,
		self.ISO639_language_code,
		self.number_segments,
		data_bytes
		)


######################################################################

class multiple_string_structure(DVBobject):

    def pack(self):
	
	
	data_bytes = string.join(
		map(lambda x: x.pack(),
		self.string_loop),
		"")

	self.number_strings = len(self.string_loop)
	
	fmt = "!B%ds" % len(data_bytes)
	return pack(fmt,
		self.number_strings,
		data_bytes,
		)
	


#! /usr/bin/env python

#
# Copyright (C) 2004-2013  Lorenzo Pallara, l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA


import string
from dvbobjects.utils import *
from dvbobjects.MPEG.Descriptor import Descriptor

######################################################################
class STD_descriptor(Descriptor):

    descriptor_tag = 0x11

    def bytes(self):
        fmt = "!B"
        return pack(fmt,
                    0xFE | self.leal_valid_flag,
                    )
		    
######################################################################
class video_stream_descriptor(Descriptor):

    descriptor_tag = 0x2
    
    def bytes(self):
	if self.MPEG_1_only_flag == 0:
	    fmt = "!B"
	    return pack(fmt,
		(self.multiple_frame_rate_flag & 0x1) << 7 |
		(self.frame_rate_code & 0x3) << 3 |
		(self.MPEG_1_only_flag & 0x1) << 2 |
		(self.constrained_parameter_flag & 0x1) << 1 |
		(self.still_picture_flag & 0x1),
	    )
	else :
	    fmt = "!BBB"
	    return pack(fmt,
		(self.multiple_frame_rate_flag & 0x1) << 7 | 
		(self.frame_rate_core & 0xF) << 3 | 
		(self.MPEG_1_only_flag & 0x1) << 2 | 
		(self.constrained_parameter_flag & 0x1) << 1 | 
		(self.still_picture_flag & 0x1),
		profile_and_level_indication,
		(chroma_format & 0x3) << 6 | 
		(frame_rate_extension_flag & 0x1) << 5 |
		0x1F,
	    )
							    

######################################################################
class audio_stream_descriptor(Descriptor):

    descriptor_tag = 0x3
    
    def bytes(self):
	fmt = "!B"
	return pack(fmt,
		(self.free_format_flag & 0x1) << 7 |
		(self.ID & 0x1) << 6 |
		(self.layer & 0x3) << 4 |
		(self.variable_rate_audio_indicator & 0x1) << 3,
	)

######################################################################
class association_tag_descriptor(Descriptor):

    descriptor_tag = 0x14

    def bytes(self):
	if self.use == 0:
    	    fmt = "!HHBLL%ds" % (len(self.private_data))
    	    return pack(fmt,    
		    self.association_tag,
		    self.use,
		    self.selector_lenght,
		    self.transaction_id,
		    self.timeout,
		    self.private_data,		    
                    )		
	else:
    	    fmt = "!HHB%ds%ds" % (
		    len(self.selector_lenght), 
		    len(self.privatedata)
		    )	    
	    return pack(fmt,
                    self.association_tag,
		    self.use,
		    self.selector_lenght,
		    self.selector_bytes,
		    self.privatedata,
		    )

######################################################################
class graphics_constraints_descriptor(Descriptor):

    descriptor_tag = 0x14
    
    def bytes(self):
	gc_bytes = string.join(
	map(lambda byte, self=self: pack("!B", byte),
		self.graphics_configuration_bytes),
		"")
		
	fmt = "!B%ds" % (len(gc_bytes))
	return pack(fmt,
		0xF8 |
		(self.can_run_without_visible_ui << 2) |
		(self.handles_configuration_changed << 1) |
		(self.handles_externally_controlled_video),
		gc_bytes,
	)

######################################################################
class DTS_registration_descriptor(Descriptor):

    descriptor_tag = 0x05

    def bytes (self):
	fmt = "!%ds" % (len(self.format_identifier))
	return pack(fmt, self.format_identifier)

######################################################################
class carousel_identifier_descriptor(Descriptor):

    descriptor_tag = 0x13

    def bytes(self):
	if self.format_ID:
        	fmt = "!LBBHHHBLBB%ds%ds" % (len(self.object_key_data) , len(self.private_data))
        	return pack(fmt,
    			self.carousel_ID,
	        	self.format_ID,
			self.module_version,
			self.module_ID,
			self.block_size,
			self.module_size,
			self.compression_method,
			self.original_size,
			self.timeout,
			len(self.object_key_data),
			self.object_key_data,
			self.private_data,
			)
	else :
                fmt = "!LB%ds" % (len(self.private_data))
                return pack(fmt,
                        self.carousel_ID,
                        self.format_ID,
                        self.private_data,
                        )
			
######################################################################							
class ca_descriptor(Descriptor):

    descriptor_tag = 0x9
    
    def bytes(self):
        fmt = "!HH"
        return pack(fmt,
            self.CA_system_ID,
            0xE000 | (self.CA_PID & 0x1FFF),
        )
        



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright 2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from dvbobjects.utils import *
from dvbobjects.DSMCC.BIOP import Tap

######################################################################
class IOR(DVBobject):

    def __init__(self, **kwargs):
        apply(DVBobject.__init__, (self,), kwargs)

        # Hard coded; TBD...
        objectLocation = BIOP_ObjectLocation(
            carouselId = self.carouselId,
            moduleId   = self.moduleId,
            objectKey  = self.objectKey,
            )

        tap = Tap.delivery_para_tap(
            assocTag      = self.assocTag,
            transactionId = self.transactionId,
            timeout       = self.timeout,
            )

        connBinder = DSM_ConnBinder(
            tap = tap,
            )

        self.profile = BIOPProfileBody(
            objectLocation = objectLocation,
            connBinder = connBinder,
            )
            
    def pack(self):

        taggedProfiles_count = 1        # MHP / TBD

        assert len(self.type_id) == 4, `self.type_id`

        profile_bytes = self.profile.pack()

        FMT = ("!"
               "L%ds"                   # type_id
               "L"                      # taggedProfiles_count
               "%ds"                    # IOP::taggedProfile()
               ) % (
            len(self.type_id),
            len(profile_bytes),
            )

        ior_bytes = pack(
            FMT,
            len(self.type_id),
            self.type_id,
            taggedProfiles_count,
            profile_bytes,
            )

        return ior_bytes

######################################################################
class BIOPProfileBody(DVBobject):

    profileId_tag = 0x49534F06
    profile_data_byte_order = 0x00

    def __init__(self, **kwargs):

        # Initialize SuperClass
        apply(DVBobject.__init__, (self,), kwargs)


    def pack(self):

        profile_data = (
            self.objectLocation.pack()
            + self.connBinder.pack()
            )
        
        lite_component_count = 2
        profile_data_length = len(profile_data) + 2

        FMT = ("!"
               "L"                      # profileId_tag
               "L"                      # profile_data_length
               "B"                      # profile_data_byte_order
               "B"                      # lite_component_count
               "%ds"                    # profile_data
               ) % (
            len(profile_data),
            )

        return pack(
            FMT,
            self.profileId_tag,
            profile_data_length,
            self.profile_data_byte_order,
            lite_component_count,
            profile_data,
            )


######################################################################
class BIOP_ObjectLocation(DVBobject):

    componentId_tag    = 0x49534F50
    biop_version_major = 0x01
    biop_version_minor = 0x00
    
    def pack(self):

        FMT = (
            "!"
            "L"                         # componentId_tag
            "B"                         # component_data_length
            "L"                         # carouselId
            "H"                         # moduleId
            "B"                         # major version
            "B"                         # minor version
            "B"                       	# objectKey length
	    "L"				# objectKey
            )  

        return pack(
            FMT,
            self.componentId_tag,
            calcsize(FMT) - 5,
            self.carouselId,
            self.moduleId,
            self.biop_version_major,
            self.biop_version_minor,
	    4,
	    self.objectKey,
            )

######################################################################
class DSM_ConnBinder(DVBobject):

    componentId_tag = 0x49534F40
    
    def pack(self):

        taps_count = 1
        taps_bytes = self.tap.pack()

        FMT = (
            "!"
            "L"                         # componentId_tag
            "B"                         # component_data_length
            "B"                         # taps_count
            "%ds"                       # taps_bytes
            ) % len(taps_bytes)

        return pack(
            FMT,
            self.componentId_tag,
            calcsize(FMT) - 5,          # component_data_length
            taps_count,
            taps_bytes,
            )



#! /usr/bin/env python

#
# Copyright (C) 2004-2013  Lorenzo Pallara, l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils import *
from dvbobjects.utils.MJD import *
from dvbobjects.MPEG.Descriptor import Descriptor

######################################################################                                                                                                                 
class short_event_descriptor(Descriptor):                                                                                                                                              

    descriptor_tag = 0x4D
    
    def bytes(self):
	assert len(self.ISO639_language_code) == 3
	fmt = "!%dsB%dsB%ds" % (
	    len(self.ISO639_language_code),
	    len(self.event_name),
	    len(self.text),
	)
	return pack(fmt,
	    self.ISO639_language_code,
	    len(self.event_name),
	    self.event_name,
	    len(self.text),
	    self.text,
	)

######################################################################
class content_user_loop_item(DVBobject):

    def pack(self):
	fmt = "!BB"
	return pack(fmt,
	    ((self.content_nibble_level_1 & 0xF) << 4)|
	    ((self.content_nibble_level_2 & 0xF)),
	    ((self.user_nibble_level_1 & 0xF) << 4)|
	    ((self.user_nibble_level_2 & 0xF))
	)


class content_descriptor(Descriptor):

    descriptor_tag = 0x54

    def bytes(self):
	data_bytes = string.join(
		map(lambda x: x.pack(),
		self.content_user_loop),
		"")

	fmt = "!%ds" % len(data_bytes)
	return pack(fmt,
	    data_bytes,
	)

######################################################################
class parental_rating_descriptor(Descriptor):                                                                                                                                              

    descriptor_tag = 0x55
    
    def bytes(self):
	assert len(self.country_code) == 3
	fmt = "!%dsB" % (
	    len(self.country_code),
	)
	return pack(fmt,
	    self.country_code,
	    self.rating,
	)

######################################################################
class dts_registration_descriptor(Descriptor):

    descriptor_tag = 0x05

    def bytes(self):
	fmt = "!L" 
	return pack(fmt,
	    self.format_identifier,
		)
#DTS format_identifier is 0x44545331 ("DTS1") for frame size 512;
#DTS format_identifier is 0x44545332 ("DTS2") for frame size 1 024;
#DTS format_identifier is 0x44545333 ("DTS3") for frame size 2 048.

######################################################################
class teletext_descriptor(Descriptor):

    descriptor_tag = 0x56

    def bytes(self):
	data_bytes = string.join(
		    map(lambda x: x.pack(),
		    self.teletext_descriptor_loop),
		    "")
		    
	fmt = "!%ds" % len(data_bytes)
	return pack(fmt,
		data_bytes,
		)

class teletext_descriptor_loop_item(DVBobject):

    def pack(self):
	assert len(self.ISO639_language_code) == 3
	fmt = "!%dsBB" % len(self.ISO639_language_code)
	return pack(fmt,
	    self.ISO639_language_code,
	    (self.type << 3) | 
	    (self.magazine_number & 0x07),
	    self.page_number,
		)

######################################################################
class vbi_data_descriptor_loop_item(DVBobject):

    def pack(self):
            fmt = "!B"
	    return pack(fmt,
		0x3F & 
		((self.field_parity & 0x01) >> 5) | 
		(self.line_offset & 0x1F),    
		)

class vbi_data_descriptor(Descriptor):

    descriptor_tag = 0x45

    def bytes(self):
	data_bytes = string.join(
		    map(lambda x: x.pack(),
		    self.vbi_data_descriptor_loop),
		    "")
		    
	fmt = "!BB%ds" % len(data_bytes)
	return pack(fmt,
	    self.data_service_id,
	    len(data_bytes),
	    data_bytes,
		)

######################################################################
class stream_identifier_descriptor(Descriptor):

    descriptor_tag = 0x52

    def bytes(self):
	fmt = "!B"
	return pack(fmt,
		    self.component_tag,
		)
		
######################################################################
class bouquet_name_descriptor(Descriptor):

    descriptor_tag = 0x71

    def bytes(self):
        fmt = "!%ds" % len(self.bouquet_name)
        return pack(fmt,
    		self.bouquet_name)


######################################################################
class data_broadcast_id_descriptor(Descriptor):

    descriptor_tag = 0x66

    def bytes(self):
	if (self.data_broadcast_ID == 0x000A): # DVB-SSU
	
	    oui_data_bytes = string.join(
		map(lambda x: x.pack(),
		self.OUI_info_loop),
		"")
	
	    oui_data_length = len(oui_data_bytes);

    	    FMT = "!HB%ds%ds" % (oui_data_length,len(self.private_data_bytes))
    	    return pack(FMT,         
                    self.data_broadcast_ID,
		    oui_data_length,
		    oui_data_bytes,
		    self.private_data_bytes,
                    )            

	else:
    	    FMT = "!H%ds" % len(self.ID_selector_bytes)
    	    return pack(FMT,         
                    self.data_broadcast_ID,
                    self.ID_selector_bytes,
                    )            
                                 
######################################################################

class local_time_offset_loop_item(DVBobject):

	def pack(self):
	
		FMT = "!%dsBBBHBBBBB" % len(self.ISO_639_language_code)
		return pack(FMT,
			self.ISO_639_language_code,
			((self.country_region_id & 0x3F) << 2) | 0x2 | (self.local_time_offset_polarity & 0x1),
			self.local_time_offset_hour,
			self.local_time_offset_minute,
			MJD_convert(self.year_of_change, self.month_of_change, self.day_of_change),
			self.hour_of_change,
			self.minute_of_change,
			self.second_of_change,
			self.next_time_offset_hour,
			self.next_time_offset_minute,
			)


######################################################################

class local_time_offset_descriptor(Descriptor):

	descriptor_tag = 0x58

	def bytes(self):
		lt_bytes = string.join(
	    map(lambda x: x.pack(),	
                self.local_time_offset_loop),
            "")								    
		FMT = "!%ds" % len(lt_bytes)
		return pack(FMT,
			lt_bytes
			)


######################################################################

class _broadcast_id_descriptor(Descriptor):

    descriptor_tag = 0x66

    def bytes(self):
        FMT = "!H%ds" % len(self.ID_selector_bytes)
        return pack(FMT,         
                    self.data_broadcast_ID,
                    self.ID_selector_bytes,
                    )            
                                 
######################################################################
class application_signalling_descriptor(Descriptor):

    descriptor_tag = 0x6F
    application_type = -1

    def bytes(self):
	if (self.application_type == -1):
	    fmt = "!"
    	    return pack(fmt)
	else:
    	    FMT = "!HB"
    	    return pack(FMT, 
		self.application_type,
		0xE0 | (self.AIT_version & 0x1F),
                )
		    
######################################################################
class network_descriptor(Descriptor):

    descriptor_tag = 0x40

    def bytes(self):
        fmt = "!%ds" % len(self.network_name)
        return pack(fmt,
                    self.network_name,
                    )
		    
######################################################################
class service_descriptor(Descriptor):

    descriptor_tag = 0x48

    def bytes(self):
        fmt = "!BB%dsB%ds" % (len(self.service_provider_name), len(self.service_name))
        return pack(fmt,
		    self.service_type,
		    len(self.service_provider_name),
		    self.service_provider_name,
		    len(self.service_name),
		    self.service_name,
                    )


######################################################################
class transport_stream_terrestrial_descriptor(Descriptor):

   descriptor_tag = 0x5a

   def bytes(self):
       fmt = "!LBBBL"
       return pack(fmt,
           self.center_frequency,
           (self.bandwidth << 5) | 0x1f,
           (self.constellation << 6) | (self.hierarchy_information << 3)| (self.code_rate_HP_stream),
           (self.code_rate_LP_stream << 5) | (self.guard_interval << 3) | (self.transmission_mode << 1) | self.other_frequency_flag,
           0xffffffff,
           )

######################################################################
class transport_stream_sat_descriptor(Descriptor):

    descriptor_tag = 0x43
 
    def bytes(self):
    	fmt = "!LHBL"
	return pack(fmt,
		self.frequency,
		self.orbital_position,
		(self.west_east_flag << 7) | (self.polarization << 5) | self.modulation,
		(self.symbol_rate << 4)| self.FEC_inner,
	)

#######################################################################
class transport_stream_cable_descriptor(Descriptor):

    descriptor_tag = 0x44
 
    def bytes(self):
	fmt = "!LHBL"
	return pack(fmt,
	    self.frequency,
	    (self.FEC_outer) | 0xFFF0,
	    (self.modulation),
	    (self.symbol_rate << 4)| (self.FEC_inner),
	)
 		    
######################################################################
class service_descriptor_loop_item(DVBobject):

    def pack(self):
	fmt = "!HB"
	return pack(fmt,
	    self.service_ID,
	    self.service_type,
	)

class service_list_descriptor(Descriptor):

    descriptor_tag = 0x41

    def bytes(self):
        dvb_service_bytes = string.join(
	    map(lambda x: x.pack(),	
                self.dvb_service_descriptor_loop),
            "")								    

        FMT = "!%ds" % len(dvb_service_bytes)
        return pack(FMT,
                    dvb_service_bytes,
                    )
                    
######################################################################
class registration_descriptor(Descriptor):

    descriptor_tag = 0x05

    def bytes(self):
        fmt = "!%ds" % len(self.format_identifier)
        return pack(fmt,
			self.format_identifier
        )

######################################################################
class lcn_service_descriptor_loop_item(DVBobject):

    def pack(self):
	fmt = "!HH"
	return pack(fmt,
	    self.service_ID,
	    ((self.visible_service_flag << 15) | 0x7C00 | self.logical_channel_number),
	)

class logical_channel_descriptor(Descriptor):

    descriptor_tag = 0x83

    def bytes(self):
        lcn_service_bytes = string.join(
	    map(lambda x: x.pack(),	
                self.lcn_service_descriptor_loop),
            "")								    

        FMT = "!%ds" % len(lcn_service_bytes)
        return pack(FMT,
                    lcn_service_bytes,
                    )

####################################################################
class logical_channel_descriptor_v2(Descriptor):

   descriptor_tag = 0x87

   def bytes(self):
       lcn_service_bytes = string.join(
       map(lambda x: x.pack(),                  self.lcn_service_descriptor_loop),
           "")                                  
       FMT = "!BB%ds%dsB%ds" % (len(self.channel_list_name), len(self.country_code), len(lcn_service_bytes))
       return pack(FMT,
               self.channel_list_id,
        	len(self.channel_list_name),
               self.channel_list_name,
               self.country_code,
               len(lcn_service_bytes),
                   lcn_service_bytes,
        )

######################################################################
class component_descriptor(Descriptor):

    descriptor_tag = 0x50

    def bytes(self):
        fmt = "!BBB%ds%ds" % (
			len(self.ISO_639_language_code),
			len(self.text_char),
			)
        return pack(fmt,
                    0xF0 | (self.stream_content),
                    self.component_type,
		    self.component_tag,
		    self.ISO_639_language_code,
		    self.text_char,
        )

######################################################################
class PDC_descriptor(Descriptor):

    descriptor_tag = 0x69

    def bytes(self):
        fmt = "!BH"
        return pack(fmt,
                    (0xF0) | (self.day >> 1),
		    (self.day << 15) | (self.month << 11) | (self.hour << 6) | (self.minute),
        )
					

#
# Copyright (C) 2004  Andreas Berger, berger@ftw.at
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


class ip_mac_platform_name_descriptor(Descriptor):
    
    descriptor_tag = 0x0c
    
    def bytes(self):
    
        fmt = "!3s%ds" % len(self.text_char_bytes)
    
        return pack(fmt,
                    self.ISO_639_language_code,
                    self.text_char_bytes
                    )
    
class ip_mac_platform_provider_name_descriptor(Descriptor):
    
    descriptor_tag = 0x0d
    
    def bytes(self):
    
        fmt = "!3s%ds" % len(self.text_char_bytes)
    
        return pack(fmt,
                    self.ISO_639_language_code,
                    self.text_char_bytes
                    )

class target_serial_number_descriptor(Descriptor):
    
    descriptor_tag = 0x08
    
    def bytes(self):
    
        fmt = "!%ds" % len(serial_data_bytes)
    
        return pack(fmt,
                    self.serial_data_bytes
                    )

class target_smartcard_descriptor(Descriptor):
    
    descriptor_tag = 0x06
    
    def bytes(self):
    
        fmt = "!I%ds" % len(self.private_data_bytes)
    
        return pack(fmt,
                    self.super_CA_system_id,
                    self.private_data_bytes
                    )                    

class subtitling_data_descriptor_loop_item(DVBobject):

    def pack(self):
    
        fmt = "!%dsBHH" % len(self.ISO_639_language_code)
        return pack(fmt,
                    self.ISO_639_language_code,
                    self.subtitling_type,
                    self.composition_page_id,
                    self.ancillary_page_id,
                    )


class subtitling_descriptor(Descriptor):
    
    descriptor_tag = 0x59

    def bytes(self):
	data_bytes = string.join(
		    map(lambda x: x.pack(),
		    self.subtitling_data_descriptor_loop),
		    "")
		    
	fmt = "!B%ds" % len(data_bytes)
	return pack(fmt,
	    len(data_bytes),
	    data_bytes,
		)


class target_MAC_address_descriptor(Descriptor):
    
    descriptor_tag = 0x07
    
    def bytes(self):
    
        fmt = "!6s%ds" % len(self.mac_addr_bytes)
    
        return pack(fmt,
                    self.mac_addr_mask,
                    self.mac_addr_bytes
                    ) 

class target_MAC_address_range_descriptor(Descriptor):
    
    descriptor_tag = 0x0e
    
    def bytes(self):
    
        fmt = "!6s%ds" % len(self.mac_addr_bytes)
    
        return pack(fmt,
                    self.mac_addr_mask,
                    self.mac_addr_bytes
                    )   

class target_IP_address_descriptor(Descriptor):
    
    descriptor_tag = 0x09
    
    def bytes(self):
    
        fmt = "!I%dI" % len(self.IPv4_addr_bytes)
    
        return pack(fmt,
                    self.IPv4_addr_mask,
                    self.IPv4_addr_bytes
                    )   
 
class target_IP_slash_descriptor(Descriptor):
    
    descriptor_tag = 0x0f
    
    def bytes(self):
    
        fmt = "!4BB"
    
        return pack(fmt,
                    self.IPv4_addr[0],
                    self.IPv4_addr[1],
                    self.IPv4_addr[2],
                    self.IPv4_addr[3],
                    self.IPv4_slash_mask
                    )   

class target_IP_source_slash_descriptor(Descriptor):
    
    descriptor_tag = 0x10
    
    def bytes(self):
    
        fmt = "!%ds" % len(self.IPv4_source_dest_bytes)
    
        return pack(fmt,
                    self.IPv4_source_dest_bytes
                    )   

class target_IPv6_address_descriptor(Descriptor):
    
    descriptor_tag = 0x0a
    
    def bytes(self):
    
        fmt = "!7s%ds" % len(self.IPv6_address_bytes)
    
        return pack(fmt,
                    self.IPv6_address_mask,
                    self.IPv6_address_bytes
                    )   

class target_IPv6_slash_descriptor(Descriptor):
    
    descriptor_tag = 0x11
    
    def bytes(self):
    
        fmt = "!%ds" % len(self.IPv6_bytes)
    
        return pack(fmt,
                    self.IPv6_bytes
                    )   

class target_IPv6_source_slash_descriptor(Descriptor):
    
    descriptor_tag = 0x12
    
    def bytes(self):
    
        fmt = "!%ds" % len(self.IPv6_source_dest_bytes)
    
        return pack(fmt,
                    self.IPv6_source_dest_bytes
                    )   

class ip_mac_stream_location_descriptor(Descriptor):
    
    descriptor_tag = 0x13
    
    def bytes(self):
    
        fmt = "!HHHHB"
    
        return pack(fmt,
                    self.network_id,
                    self.original_network_id,
                    self.transport_stream_id,
                    self.service_id,
                    self.component_tag
                    )

class isp_access_mode_descriptor(Descriptor):
    
    descriptor_tag = 0x14
    
    def bytes(self):
    
        fmt = "!B"
    
        return pack(fmt,
                    self.access_mode
                    )

class telephone_descriptor(Descriptor):
    
    descriptor_tag = 0x57
    
    def bytes(self):
    
        fmt = "!BBBB%ds%ds%ds%ds%ds" % (len(country_prefix_bytes), len(international_area_code_bytes), len(operator_code_bytes), len(national_area_code_bytes), len(core_number_bytes))
    
        return pack(fmt,
                    (0x03 << 7) & 0xC0 | (self.foreign_availability << 5) & 0x20 | self.connection_type & 0x1F,
                    (0x01 << 7) & 0x80 | (self.country_prefix_length << 5) & 0x60 | (self.international_area_code << 4) & 0x1C | self.operator_code_length & 0x07,
                    (0x01 << 7) & 0x80 | (self.national_area_code_length << 4) & 0x70 | self.core_number_length & 0x0F, 
                    country_prefix_bytes,
                    international_area_code_bytes,
                    operator_code_bytes,
                    national_area_code_bytes,
                    core_number_bytes
                    )
  
class private_data_specifier_descriptor(Descriptor):
    
    descriptor_tag = 0x5f
    
    def bytes(self):
    
        fmt = "!I"
    
        return pack(fmt,
                    self.private_data_specifier
                    )

class time_slice_fec_identifier_descriptor(Descriptor):
    
    descriptor_tag = 0x77

    def bytes(self):

    	time_slice_fec_id = 0x00;

        fmt = "!BBB"

        return pack(fmt,
                    (self.time_slicing << 7) & 0x80 | (self.mpe_fec << 5) & 0x60 | (0x03 << 3) | self.frame_size & 0x07,
                    self.max_burst_duration,
                    (self.max_average_rate << 4) & 0xF0 | time_slice_fec_id & 0x0F,
                    )

# FIXME: move this class to another file, it's no descriptor
class platform_id_data2(DVBobject):
  
  def pack(self):
	fmt = "!BBBBB"

	return pack(fmt,
		    (self.platform_id >> 16) & 0xFF,
		    (self.platform_id >> 8) & 0xFF,
		    self.platform_id  & 0xFF,
		    self.action_type & 0xFF,
		    (0x03 << 6) & 0xC0 | (0x00 << 5) & 0x20 | 0x01 & 0x1F
		    )
    

# FIXME: move this class to another file, it's no descriptor
class ip_mac_notification_info(DVBobject):

  def pack(self):
	  
    # pack platform id data loop
    pid_bytes = string.join(
      map(lambda x: x.pack(),
	self.platform_id_data_loop),
      "")

    platform_id_data_length = len(pid_bytes);

    fmt = "!B%ds%ds" % (platform_id_data_length, len(self.private_data_bytes))

    return pack(fmt,
		platform_id_data_length,
		pid_bytes,
		self.private_data_bytes
		)

# FIXME: move this class to another file, it's no descriptor
class platform_name(DVBobject):
  
  def pack(self):
	platform_name_length = len(self.text_char_bytes)

	fmt = "!3sB%ds" % platform_name_length

	return pack(fmt,
		    self.ISO_639_language_code,
		    platform_name_length,
		    self.text_char_bytes
		    )

# FIXME: move these classes to another file, they are no exactly descriptors
class platform_id_data(DVBobject):

  def pack(self):
	  
	pn_bytes = string.join(
	  map(lambda x: x.pack(),
	    self.platform_name_loop),
          "")

	platform_name_loop_length = len(pn_bytes)

	fmt = "!BBBB%ds" % platform_name_loop_length

	return pack(fmt,
		    (self.platform_id >> 16) & 0xFF,
		    (self.platform_id >> 8) & 0xFF,
		    self.platform_id  & 0xFF,
		    platform_name_loop_length,
		    pn_bytes
		    )

class OUI_data(DVBobject):

  def pack(self):
  
	fmt = "!HBB%ds" % len(self.selector_bytes)

	return pack(fmt,
		    self.OUI >> 8,
		    self.OUI & 0xFF,
		    len(self.selector_bytes),
		    self.selector_bytes,
		    )

class OUI_info_loop_item(DVBobject):

  def pack(self):
  
	fmt = "!HBBBB%ds" % len(self.selector_bytes)

	return pack(fmt,
		    self.OUI >> 8,
		    self.OUI & 0xFF,
		    0xF0 | self.update_type & 0xF,
		    0xC0 | (self.update_versioning_flag & 0x1) << 5 | self.update_version & 0x1F,
		    len(self.selector_bytes),
		    self.selector_bytes,
		    )

class compatibility_descriptor_loop_item(DVBobject):

  def pack(self):
	
	number_descriptors = len(self.compatibility_descriptor_subloop)
	
	compatibility_descriptor_subloop_bytes = string.join(
	  map(lambda x: x.pack(),
	    self.compatibility_descriptor_subloop),
          "")

	fmt = "!BBBBBBHHB%ds" % len(compatibility_descriptor_subloop_bytes)

	return pack(fmt,
		    self.descriptor_type,
		    9 + len(compatibility_descriptor_subloop_bytes),
		    self.specifier_type,
		    (self.specifier_data >> 16) & 0xFF,
		    (self.specifier_data >> 8) & 0xFF,
		    self.specifier_data & 0xFF,
		    self.model,
		    self.version,
		    number_descriptors,
		    compatibility_descriptor_subloop_bytes,
		    )


class compatibility_descriptor(DVBobject):

  def pack(self):
  
 	number_descriptors = len(self.compatibility_descriptor_loop)
 	
	compatibility_descriptor_loop_bytes = string.join(
	  map(lambda x: x.pack(),
	    self.compatibility_descriptor_loop),
          "")
          
	fmt = "!H%ds" % len(compatibility_descriptor_loop_bytes)
	
	return pack(fmt,
		    number_descriptors,
		    compatibility_descriptor_loop_bytes,
		    )


class linkage_descriptor(Descriptor):
    
    descriptor_tag = 0x4A;

    def bytes(self):

	if (self.linkage_type == 0x0B):
        
	  # pack platform id data loop
	  pid_bytes = string.join(
          map(lambda x: x.pack(),
	    self.platform_id_data_loop),
          "")

	  platform_id_data_length = len(pid_bytes);

	  fmt = "!BBBBBBBB%ds%ds" % (platform_id_data_length, len(self.private_data_bytes))

	  return pack(fmt,
		    (self.transport_stream_id >> 8) & 0xFF,
		    self.transport_stream_id & 0xFF,
		    (self.original_network_id >> 8) & 0xFF,
		    self.original_network_id & 0xFF,
		    (self.service_id >> 8) & 0xFF,
		    self.service_id & 0xFF,
		    self.linkage_type,
		    platform_id_data_length,
		    pid_bytes,
		    self.private_data_bytes
		    )
	
	elif (self.linkage_type == 0x09):

	  # pack oui data loop
	  oui_data_bytes = string.join(
          map(lambda x: x.pack(),
	    self.OUI_loop),
          "")

	  oui_data_length = len(oui_data_bytes);

	  fmt = "!BBBBBBBB%ds%ds" % (oui_data_length, len(self.private_data_bytes))

	  return pack(fmt,
		    (self.transport_stream_id >> 8) & 0xFF,
		    self.transport_stream_id & 0xFF,
		    (self.original_network_id >> 8) & 0xFF,
		    self.original_network_id & 0xFF,
		    (self.service_id >> 8) & 0xFF,
		    self.service_id & 0xFF,
		    self.linkage_type,
		    oui_data_length,
		    oui_data_bytes,
		    self.private_data_bytes
		    )

	elif (self.linkage_type == 0x0A):

	  fmt = "!BBBBBBBB%ds" % len(self.private_data_bytes)

	  return pack(fmt,
		    (self.transport_stream_id >> 8) & 0xFF,
		    self.transport_stream_id & 0xFF,
		    (self.original_network_id >> 8) & 0xFF,
		    self.original_network_id & 0xFF,
		    (self.service_id >> 8) & 0xFF,
		    self.service_id & 0xFF,
		    self.linkage_type,
		    self.table_type,
		    self.private_data_bytes
		    )

	else:
	  fmt = "!BBBBBBB%ds" % len(self.private_data_bytes)

	  # we care only for some linkage_type, other linkage descriptors
	  # have to be implemented according to ETSI EN 300 468 standard

	  return pack(fmt,
		    (self.transport_stream_id >> 8) & 0xFF,
		    self.transport_stream_id & 0xFF,
		    (self.original_network_id >> 8) & 0xFF,
		    self.original_network_id & 0xFF,
		    (self.service_id >> 8) & 0xFF,
		    self.service_id & 0xFF,
		    self.linkage_type,
		    self.private_data_bytes
		    )

class terrestrial_delivery_system_descriptor(Descriptor):

    descriptor_tag = 0x5a

    def bytes(self):
        fmt = "!LBBBL"
	priority = 1
	timeslice_ind = 1
	mpe_fec_ind = 1
        return pack(fmt,
                    self.center_frequency & 0xFFFFFFFF,
		    (self.bandwidth << 5) & 0xE0 | (self.priority << 4) & 0x10 | (self.timeslice_ind << 3) & 0x08 |
		    (self.mpe_fec_ind << 2) & 0x04 | 0x03,
		    (self.constellation << 6) | (self.hierarchy_information << 3)| (self.code_rate_HP_stream),
		    (self.code_rate_LP_stream << 5) | (self.guard_interval << 3) | (self.transmission_mode << 1) | (self.other_frequency_flag),
		    0xffffffff,
                    )



######################################################################                                                                                                                 
class crid_descriptor_loop_item(DVBobject):

    def pack(self):

	if (self.location == 0x0):
    
    	    fmt = "!BB%ds" % len(self.crid)
	    return pack(fmt,
		(self.type & 0x3F) << 2 |
		(self.location),
	        len(self.crid),
	        self.crid,
	    )
	
	else:
	    fmt = "!BH"
	    return pack(fmt,
		(self.type & 0x3F) << 2 |
		(self.location & 0x3),
	        self.crid_ref,
	    )

class content_identifier_descriptor(Descriptor):

    descriptor_tag = 0x76

    def bytes(self):
        crid_bytes = string.join(
	    map(lambda x: x.pack(),	
                self.crid_descriptor_loop),
            "")								    

        FMT = "!%ds" % len(crid_bytes)
        return pack(FMT,
                    crid_bytes,
                    )


class ssu_location_descriptor(Descriptor):

    descriptor_tag = 0x03

    def bytes(self):
    
	if (self.data_broadcast_id == 0x000A):
	
    	    FMT = "!HH%ds" % len(self.private_data_bytes)
    	    return pack(FMT,
    		    self.data_broadcast_id,
    		    self.association_tag,
                    self.private_data_bytes,
                    )

        else:
    	    FMT = "!H%ds" % len(self.private_data_bytes)
    	    return pack(FMT,
    		    self.data_broadcast_id,
                    self.private_data_bytes,
                    )

class scheduling_descriptor(Descriptor):

    descriptor_tag = 0x01

    def bytes(self):
    
	FMT = "!HBBBHBBBBBBB%ds" % len(self.private_data_bytes)
    	return pack(FMT,
    		MJD_convert(self.year_of_start_date_time, self.month_of_start_date_time, self.day_of_start_date_time),
		self.hour_of_start_date_time,
		self.minute_of_start_date_time,
		self.second_of_start_date_time,
    		MJD_convert(self.year_of_end_date_time, self.month_of_end_date_time, self.day_of_end_date_time),
		self.hour_of_end_date_time,
		self.minute_of_end_date_time,
		self.second_of_end_date_time,
    		self.final_availability << 7 | self.Periodicity_flag << 6 | self.period_unit << 4 | self.duration_unit << 2 | self.estimated_cycle_time_unit,
    		self.Period,
    		self.Duration,
    		self.estimated_cycle_time,
                self.private_data_bytes,
                )

######################################################################
class default_authority_descriptor(Descriptor):

    descriptor_tag = 0x73
    
    def bytes(self):
        fmt = "!%ds" % len(self.authority)
        return pack(fmt,
                    self.authority,
                    )

######################################################################                                                                                                                 
class extended_event_loop_item(DVBobject):

    def pack(self):
    
	fmt = "!B%dsB%ds" % (len(self.item_description), len(self.item))
	return pack(fmt,
	    len(self.item_description),
	    self.item_description,
	    len(self.item),
	    self.item,
	)

class extended_event_descriptor(Descriptor):                                                                                                                                              

    descriptor_tag = 0x4E
    
    def bytes(self):
	item_bytes = string.join(
	    map(lambda x: x.pack(),	
                self.extended_event_loop),
            "")	        
	assert len(self.ISO639_language_code) == 3
        FMT = "!B%dsB%dsB%ds" % (len(self.ISO639_language_code), len(item_bytes), len(self.text))
        return pack(FMT,
    		    (self.descriptor_number & 0xF) << 4 |
    		    (self.last_descriptor_number & 0xF),
    		    self.ISO639_language_code,
		    len(item_bytes),
    		    item_bytes,
    		    len(self.text),
    		    self.text,
    		    )
     
######################################################################
class ac3_descriptor(Descriptor):

    descriptor_tag = 0x6A
    
    
    def bytes(self):
    	fmt = "!B"
    	flags_flag = 0
    	flags = ""
	if self.component_type_flag == 1:
	    flags = pack("!B", self.component_type)
	    flags_flag = 1
	if self.bsid_flag == 1:
	    flags = flags + pack("!B", self.bsid)
	    flags_flag = 1
	if self.mainid_flag == 1:
	    flags = flags + pack("!B", self.mainid)
	    flags_flag = 1
	if self.asvc_flag == 1:
	    flags = flags + pack("!B", self.asvc)
	    flags_flag = 1
	if flags_flag == 0 :
	    fmt = (fmt + "%ds") % len(self.additional_info)
	    return pack(fmt,
		(self.component_type_flag & 0x1) << 7 |
		(self.bsid_flag & 0x1) << 6 |
		(self.mainid_flag & 0x1) << 5 |
		(self.asvc_flag & 0x1) << 4,
		self.additional_info
		)
	else :
	    fmt = (fmt + "%ds%ds") % (len(flags), len(self.additional_info))
	    return pack(fmt,
		(self.component_type_flag & 0x1) << 7 |
		(self.bsid_flag & 0x1) << 6 |
		(self.mainid_flag & 0x1) << 5 |
		(self.asvc_flag & 0x1) << 4,
		flags,
		self.additional_info
		)
	
######################################################################
class dts_audio_stream_descriptor(Descriptor):

	descriptor_tag = 0x7B
	
	def bytes(self):
		fmt = "!BBBBB%ds" % len(self.additional_info)
		return pack(fmt,
		((self.sample_rate_code & 0xF) << 4) | ((self.bit_rate_code >> 2) & 0xF),
		((self.bit_rate_code & 0x3) << 6) | ((self.nblks >> 1) & 0x3F),
		((self.nblks & 0x1) << 7) | ((self.fsize >> 7) & 0x7F),
		((self.fsize & 0x7F) << 1) | ((self.surround_mode >> 5) & 0x1),
		((self.surround_mode & 0x1F) << 3) | ((self.lfe_flag & 0x1) << 2) | (self.extendend_surround_flag & 0x3),
		self.additional_info
		)

######################################################################
class ISO_639_language_descriptor(Descriptor):

    descriptor_tag = 0x0A

    def bytes(self):
        fmt = "!%dsB" % (
			len(self.ISO_639_language_code),
			)
        return pack(fmt,
		    self.ISO_639_language_code,
		    self.Audio_type,
        )

######################################################################
class supplementary_audio_descriptor(Descriptor):

    descriptor_tag = 0x7F;
    
    def bytes(self):
	
	descriptor_tag_extension = 0x06;
	
	fmt = "!BB%ds%ds" % (len(self.ISO_639_language_code),len(self.private_data_bytes))
	
	return pack(fmt,
		descriptor_tag_extension,
		(self.mix_type & 0x1) << 7 | (self.editorial_classification & 0x1F) << 2 | 0x3,
		self.ISO_639_language_code,
		self.private_data_bytes,
	)

######################################################################
class private_data_descriptor(Descriptor):

    descriptor_tag = 0x0F

    def bytes(self):
        fmt = "!"
        return pack(fmt)

######################################################################
class bouquet_descriptor(Descriptor):

    descriptor_tag = 0x47

    def bytes(self):
        fmt = "!%ds" % len(self.bouquet_name)
        return pack(
            fmt,
            self.bouquet_name,
        )

